package pl.slawas.scjdbc4jpa.config;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import pl.slawas.helpers.Configurations;

/**
 * 
 * JpaConfig - klasa zarządzająca parametrami uruchomienia kontenera JPA.
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.1 $
 * 
 */
public class JpaConfig {

	final protected static Logger logger = LoggerFactory
			.getLogger(JpaConfig.class);

	public static final String FILE_NAME = "application.properties";

	public static final String FILE_DIRECTORY = "";

	private static Hashtable<String, String> _Properties = null;

	private static JpaConfig _Instance;

	private boolean isInvalid = true;

	private Properties propertyList;

	/**
	 * Konstruktor domyślny z domyślną ścieżką położenia pliku konfoguracyjnego
	 * {@link #FILE_DIRECTORY}/{@link #FILE_NAME}
	 * 
	 */
	private JpaConfig() {
		loadPropertiesFromFile();
		propertyList = generatePropertyList();
	}

	/**
	 * Konstruktor ze wskazaniem innej lokalizacji położenia pliku
	 * konfiguracyjnego
	 * 
	 * @param configFileName
	 *            pełna ścieżka pliku konfiguracyjnego
	 */
	private JpaConfig(String configFileName) {
		loadPropertiesFromFile(configFileName);
		propertyList = generatePropertyList();
	}

	/**
	 * Metoda ladująca właściwości z pliku
	 * 
	 * @throws Exception
	 */
	synchronized private void loadPropertiesFromFile() {
		loadPropertiesFromFile(FILE_DIRECTORY + "/" + FILE_NAME);
	}

	/**
	 * Metoda ladująca właściwości z pliku
	 * 
	 * @throws Exception
	 */
	synchronized private void loadPropertiesFromFile(String configFileName) {

		logger.debug("Loading configuration....");

		if (_Properties != null) {
			_Properties.clear();
		}
		_Properties = Configurations.loadHashtable(JpaConfig.class,
				configFileName);

	}

	private synchronized Properties generatePropertyList() {

		this.propertyList = new Properties();
		Enumeration<String> props = _Properties.keys();
		while (props.hasMoreElements()) {
			String key = props.nextElement();
			String value = _Properties.get(key);
			this.propertyList.put(key, value);
		}

		isInvalid = false;
		return this.propertyList;
	}

	/**
	 * Pobranie instancji konfiguracji
	 * 
	 * @return instancja konfiguracji klienta
	 */
	public synchronized static JpaConfig getInstance() {
		if (_Instance == null) {
			logger.debug("Get new instance.");
			_Instance = new JpaConfig();
		}
		return _Instance;
	}

	/**
	 * Pobranie instancji konfiguracji ze wskazaniem nowej lokalizacji pliku
	 * konfiguracyjnego
	 * 
	 * @param configFileName
	 *            pełna ścieżka pliku konfiguracyjnego
	 * @return instancja pliku konfiguracji
	 */
	public synchronized static JpaConfig getInstance(String configFileName) {
		if (_Instance == null) {
			logger.debug("Get new instance.");
			_Instance = new JpaConfig(configFileName);
		}
		return _Instance;
	}

	public String get(String propertyCode) {
		return (String) _Properties.get(propertyCode);
	}

	public void put(String propertyCode, String value) {
		_Properties.put(propertyCode, value);
		isInvalid = true;
	}

	public Properties getPropertyList() {
		if (this.isInvalid || this.propertyList == null)
			generatePropertyList();
		return this.propertyList;
	}

}
