package pl.slawas.scjdbc4jpa.config;

import java.sql.ResultSet;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import pl.slawas.helpers.ConfigJAXBFactory;
import pl.slawas.paging.Page;
import pl.slawas.paging.PagingParams;
import pl.slawas.scjdbc4jpa.jdbc.ResultType;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCTableDesc;

public class LoadJdbcConfiguration extends ConfigJAXBFactory<JdbcConfig> {

	final static private Logger logger = LoggerFactory
			.getLogger(LoadJdbcConfiguration.class);

	/**
	 * Maksymalna liczba stron,jaka może być zwrócona przez zapytanie JDBC
	 * zdefiniowana w pliku konfiguracyjnym parametrem
	 * {@link JpaProperties#JDBC_RESULT_MAX_PAGES}, jednak gdy nie jest on
	 * zdefiniowany przyjmuje on wartosc domyślną zdefiniowaną w
	 * {@link Page#DEFAULT_MAX_PAGES}.
	 * <p>
	 * Parametr ten implikuje ustawienie maksymalnej liczby wierszy zwracanej
	 * przez wynik zapytania:<br>
	 * {@code {@link #RESULT_MAX_PAGES} * <rozmiar_strony>}<br>
	 * gdzie {@code <rozmiar_strony>} to parametr {@code pageSize} podany w
	 * konstruktorze obiektu np.
	 * {@link #ResultRowMapper(JDBCTableDesc, ResultSet, Integer, Integer)}.<br>
	 * Jeżeli zostanie użyty kostruktor, który nie zawyera tych parametrów,
	 * rozmiar strony przyjmie wartość stałej
	 * {@link JdbcConfig#DEFAULT_PAGE_SIZE}.
	 * </p>
	 * <p>
	 * Do sterowania ograniczeniami można wykorzystać obiekt {@link ResultType}.
	 * </p>
	 * 
	 */
	public static int RESULT_MAX_PAGES = (JpaConfig.getInstance().get(
			JpaProperties.JDBC_RESULT_MAX_PAGES) == null ? Page.DEFAULT_MAX_PAGES
			: Integer.valueOf(JpaConfig.getInstance().get(
					JpaProperties.JDBC_RESULT_MAX_PAGES)));

	/**
	 * Pozycja pierwszego wiersza
	 */
	public final static int FIRST_ROW_POSITION = 1;

	/**
	 * Pobieranie obiektu z parametrami stronicowania.
	 * 
	 * @param pageSize
	 *            rozmiar strony, może być {@code null}, wtedy zostanie pobrany
	 *            {@link Page#DEFAULT_PAGE_SIZE}.
	 * @return obiekt parametrów stronicowania
	 */
	public static PagingParams getPagingParams(Integer pageSize) {
		if (pageSize == null)
			pageSize = Page.DEFAULT_PAGE_SIZE;
		PagingParams pp = new PagingParams(pageSize, 0, pageSize,
				RESULT_MAX_PAGES * pageSize);
		logger.trace("Parametry stronicowania: {}", pp.toString());
		return pp;
	}

	public LoadJdbcConfiguration(String url, boolean readConfigFromJar) {
		super(url, readConfigFromJar);
	}

}
