package pl.slawas.scjdbc4jpa.core;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Savepoint;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.transaction.SavepointManager;
import org.springframework.transaction.support.ResourceHolderSupport;
import org.springframework.util.Assert;


public class ScEntityManagerHolder extends ResourceHolderSupport {

	final protected Log log = LogFactory.getLog(getClass());

	public static final String SAVEPOINT_NAME_PREFIX = "ScSAVEPOINT_";

	private ScEntityManagerHandle entityHandle;

	private SavepointManager savepointManager;

	private boolean transactionActive = false;

	private Boolean savepointsSupported;

	private int savepointCounter = 0;

	private ScEntityManager currentEntityManager;

	private Connection currentConnection;

	public ScEntityManagerHolder(ScEntityManager entityManagerObject) {
		Assert.notNull(entityManagerObject,
				"ScEntityManagerObject must not be null");
		this.entityHandle = new ScEntityManagerHandleBean(entityManagerObject);
	}

	public ScEntityManagerHolder(ScEntityManagerHandle aemh) {
		Assert.notNull(aemh, "ScEntityManagerHandle must not be null");
		this.entityHandle = aemh;
	}

	protected void setTransactionActive(boolean transactionActive) {
		this.transactionActive = transactionActive;
	}

	protected boolean isTransactionActive() {
		return this.transactionActive;
	}

	protected void setSavepointManager(SavepointManager savepointManager) {
		this.savepointManager = savepointManager;
	}

	protected SavepointManager getSavepointManager() {
		return this.savepointManager;
	}

	public ScEntityManager getScEntityManager() {
		Assert.notNull(this.entityHandle,
				"Active ScEntityManagerHandle is required");
		if (this.currentEntityManager == null) {
			this.currentEntityManager = this.entityHandle
					.getEntityManagerObject();
		}
		return this.currentEntityManager;
	}

	public Connection getConnection() throws SQLException {
		if (this.currentConnection == null) {
			this.currentConnection = (getScEntityManager() != null ? this.currentEntityManager
					.getConnection() : null);
		}
		if (this.currentConnection == null)
			throw new SQLException("Can't get connection");

		return this.currentConnection;
	}

	public boolean supportsSavepoints() throws SQLException {
		if (this.savepointsSupported == null) {
			this.savepointsSupported = new Boolean(getConnection()
					.getMetaData().supportsSavepoints());
		}
		return this.savepointsSupported.booleanValue();
	}

	public Savepoint createSavepoint() throws SQLException {
		this.savepointCounter++;
		return getConnection().setSavepoint(
				SAVEPOINT_NAME_PREFIX + this.savepointCounter);
	}

	public void released() {
		super.released();
		try {
			this.entityHandle.releaseEntityManager();
		} catch (SQLException e) {
			log.error("Can't release ScEntityManager", e);
		}
		this.currentConnection = null;
		this.currentEntityManager = null;
	}

	public void clear() {
		super.clear();
		this.transactionActive = false;
		this.savepointManager = null;
		getScEntityManager().clear();
	}

}
