package pl.slawas.scjdbc4jpa.core;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.SQLException;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.PersistenceException;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.orm.jpa.DefaultJpaDialect;
import org.springframework.orm.jpa.EntityManagerFactoryInfo;
import org.springframework.orm.jpa.EntityManagerFactoryUtils;
import org.springframework.orm.jpa.JpaDialect;
import org.springframework.transaction.NestedTransactionNotSupportedException;
import org.springframework.transaction.SavepointManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionException;
import org.springframework.transaction.TransactionSystemException;
import org.springframework.transaction.support.AbstractPlatformTransactionManager;
import org.springframework.transaction.support.DefaultTransactionStatus;
import org.springframework.transaction.support.ResourceTransactionManager;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import pl.slawas.scjdbc4jpa.jpa._JpaEntityManagerFactory;

public class ScPlatformTransactionManager extends
		AbstractPlatformTransactionManager implements
		ResourceTransactionManager, InitializingBean, Serializable, ScDefaults {

	final protected transient Logger logger = Logger.getLogger(getClass());

	private static final long serialVersionUID = 6523153525047973947L;

	private _JpaEntityManagerFactory entityManagerFactory;

	private ScEntityManager ScEntityManager;

	private String dialectName = DefaultDialect;

	private JpaDialect jpaDialect = new DefaultJpaDialect();

	/**
	 * Create a new ScTransactionManager instance. A ScEntityManagerFactory has
	 * to be set to be able to use it.
	 * 
	 * @see #setEntityManagerFactory
	 */
	public ScPlatformTransactionManager() {
		setNestedTransactionAllowed(true);
	}

	/**
	 * Create a new ScTransactionManager instance.
	 * 
	 * @param emf
	 *            ScEntityManagerFactory to manage transactions for
	 */
	public ScPlatformTransactionManager(EntityManagerFactory emf) {
		this();
		this.entityManagerFactory = (_JpaEntityManagerFactory) emf;
		try {
			afterPropertiesSet();
		} catch (Exception e) {
			// TODO poprawić obsługę błędów
			e.printStackTrace();
		}
	}

	/**
	 * @return the entityManagerFactory
	 */
	public EntityManagerFactory getEntityManagerFactory() {
		return entityManagerFactory;
	}

	/**
	 * @param entityManagerFactory
	 *            the entityManagerFactory to set
	 */
	public void setEntityManagerFactory(
			EntityManagerFactory entityManagerFactory) {
		this.entityManagerFactory = (_JpaEntityManagerFactory) entityManagerFactory;
	}

	/**
	 * @return the dialectName
	 */
	public String getDialectName() {
		return dialectName;
	}

	/**
	 * @param dialectName
	 *            the dialectName to set
	 */
	public void setDialectName(String dialectName) {
		this.dialectName = dialectName;
	}

	public void setJpaDialect(JpaDialect jpaDialect) {
		this.jpaDialect = (jpaDialect != null ? jpaDialect
				: new DefaultJpaDialect());
	}

	/**
	 * Return the JPA dialect to use for this transaction manager.
	 */
	public JpaDialect getJpaDialect() {
		if (this.jpaDialect == null)
			this.jpaDialect = new DefaultJpaDialect();
		return this.jpaDialect;
	}

	@Override
	protected void doBegin(Object transaction, TransactionDefinition definition)
			throws TransactionException {
		if (dialectName.equals(DefaultDialect)) {

			ScTransactionObject txObject = (ScTransactionObject) transaction;
			ScEntityManager aem = null;

			try {
				if (txObject.getScEntityManagerHolder() == null
						|| txObject.getScEntityManagerHolder()
								.isSynchronizedWithTransaction()) {
					ScEntityManager newEm = createEntityManagerForTransaction();
					if (logger.isDebugEnabled()) {
						logger.debug("Opened new EntityManager [" + newEm
								+ "] for JPA transaction");
					}
					txObject.setScEntityManagerHolder(
							new ScEntityManagerHolder(newEm), true);
				}

				aem = txObject.getScEntityManagerHolder().getScEntityManager();

				// Delegate to JpaDialect for actual transaction begin.
				aem.getTransaction().begin();
				Object transactionData = null;
				txObject.setTransactionData(transactionData);

				// Register transaction timeout.
				int timeout = determineTimeout(definition);
				if (timeout != TransactionDefinition.TIMEOUT_DEFAULT) {
					txObject.getScEntityManagerHolder().setTimeoutInSeconds(
							timeout);
				}

				// Bind the entity manager holder to the thread.
				if (txObject.isNewEntityManagerHolder()) {
					TransactionSynchronizationManager.bindResource(
							getScEntityManager(),
							txObject.getScEntityManagerHolder());
				}
				txObject.getScEntityManagerHolder()
						.setSynchronizedWithTransaction(true);
			}

			catch (TransactionException ex) {
				closeEntityManagerAfterFailedBegin(txObject);
				throw ex;
				// } catch (Exception ex) {
				// closeEntityManagerAfterFailedBegin(txObject);
				// throw new CannotCreateTransactionException(
				// "Could not open JPA EntityManager for transaction", ex);
			}

		} else {
			// TODO zrobic obsluge dialectu Oracle-a
		}

	}

	@Override
	protected void doCommit(DefaultTransactionStatus status)
			throws TransactionException {
		ScTransactionObject txObject = (ScTransactionObject) status
				.getTransaction();
		if (status.isDebug()) {
			logger.debug("Committing JPA transaction on EntityManager ["
					+ txObject.getScEntityManagerHolder().getScEntityManager()
					+ "]");
		}
		try {
			EntityTransaction tx = txObject.getScEntityManagerHolder()
					.getScEntityManager().getTransaction();
			tx.commit();
		}
		// catch (RollbackException ex) {
		// if (ex.getCause() instanceof RuntimeException) {
		// DataAccessException dex =
		// getJpaDialect().translateExceptionIfPossible((RuntimeException)
		// ex.getCause());
		// if (dex != null) {
		// throw dex;
		// }
		// }
		// throw new TransactionSystemException("Could not commit JPA
		// transaction", ex);
		// }
		catch (RuntimeException ex) {
			// Assumably failed to flush changes to database.
			throw DataAccessUtils.translateIfNecessary(ex, getJpaDialect());
		}

	}

	protected boolean isExistingTransaction(Object transaction) {
		return ((ScTransactionObject) transaction).hasTransaction();
	}

	protected Object doSuspend(Object transaction) {
		ScTransactionObject txObject = (ScTransactionObject) transaction;
		txObject.setScEntityManagerHolder(null, false);
		ScEntityManagerHolder entityManagerHolder = (ScEntityManagerHolder) TransactionSynchronizationManager
				.unbindResource(getScEntityManager());
		return new SuspendedResourcesHolder(entityManagerHolder);
	}

	protected void doResume(Object transaction, Object suspendedResources) {
		logger.debug("use ScTransactionManager.doResume");
		SuspendedResourcesHolder resourcesHolder = (SuspendedResourcesHolder) suspendedResources;
		TransactionSynchronizationManager.bindResource(getScEntityManager(),
				resourcesHolder.getScEntityManagerHolder());
	}

	protected void doCleanupAfterCompletion(Object transaction) {
		ScTransactionObject txObject = (ScTransactionObject) transaction;

		// Remove the entity manager holder from the thread.
		if (txObject.isNewEntityManagerHolder()) {
			TransactionSynchronizationManager
					.unbindResource(getScEntityManager());
		}
		txObject.getScEntityManagerHolder().clear();
		getJpaDialect().cleanupTransaction(txObject.getTransactionData());

		// Remove the entity manager holder from the thread.
		if (txObject.isNewEntityManagerHolder()) {
			ScEntityManager em = txObject.getScEntityManagerHolder()
					.getScEntityManager();
			if (logger.isDebugEnabled()) {
				logger.debug("Closing JPA EntityManager [" + em
						+ "] after transaction");
			}
			EntityManagerFactoryUtils.closeEntityManager(em);
		} else {
			logger.debug("Not closing pre-bound JPA EntityManager after transaction");
		}
	}

	protected void doSetRollbackOnly(DefaultTransactionStatus status) {
		ScTransactionObject txObject = (ScTransactionObject) status
				.getTransaction();
		if (status.isDebug()) {
			logger.debug("Setting JPA transaction on EntityManager ["
					+ txObject.getScEntityManagerHolder().getScEntityManager()
					+ "] rollback-only");
		}
		txObject.setRollbackOnly();
	}

	@Override
	protected Object doGetTransaction() throws TransactionException {
		Object customObject;
		ScTransactionObject txObject = new ScTransactionObject();
		ScEntityManagerHolder emHolder = (ScEntityManagerHolder) TransactionSynchronizationManager
				.getResource((ScEntityManager) getScEntityManager());
		if (emHolder != null) {
			if (logger.isDebugEnabled()) {
				logger.debug("Found thread-bound ScEntityManager ["
						+ emHolder.getScEntityManager()
						+ "] for JPA transaction");
			}
			txObject.setScEntityManagerHolder(emHolder, false);
		}

		customObject = txObject;
		return customObject;
	}

	@Override
	protected void doRollback(DefaultTransactionStatus status)
			throws TransactionException {
		ScTransactionObject txObject = (ScTransactionObject) status
				.getTransaction();
		if (status.isDebug()) {
			logger.debug("Rolling back JPA transaction on EntityManager ["
					+ txObject.getScEntityManagerHolder().getScEntityManager()
					+ "]");
		}
		try {
			EntityTransaction tx = txObject.getScEntityManagerHolder()
					.getScEntityManager().getTransaction();
			if (tx.isActive()) {
				tx.rollback();
			}
		} catch (PersistenceException ex) {
			throw new TransactionSystemException(
					"Could not roll back JPA transaction", ex);
		} finally {
			if (!txObject.isNewEntityManagerHolder()) {
				// Clear all pending inserts/updates/deletes in the
				// EntityManager.
				// Necessary for pre-bound EntityManagers, to avoid inconsistent
				// state.
				txObject.getScEntityManagerHolder().getScEntityManager()
						.clear();
			}
		}

	}

	public Object getResourceFactory() {
		return getEntityManagerFactory();
	}

	public void afterPropertiesSet() throws Exception {
		if (getEntityManagerFactory() == null) {
			throw new IllegalArgumentException(
					"Property 'entityManagerFactory' is required");
		}
		if (!(getEntityManagerFactory() instanceof _JpaEntityManagerFactory)) {
			this.setScEntityManager(createEntityManagerForTransaction());
		} else if (!(getEntityManagerFactory() instanceof EntityManagerFactoryInfo)) {
			this.setScEntityManager(createEntityManagerForTransaction());
		}
		logger.info("\n Transaction Manager: afterPropertiesSet()\n");
		// TODO zrobic sprawdzenie czy zostal ustawiony dialekt
	}

	protected ScEntityManager createEntityManagerForTransaction() {
		return (ScEntityManager) ((_JpaEntityManagerFactory) getEntityManagerFactory())
				.createEntityManager();
	}

	/**
	 * Close the current transaction's EntityManager. Called after a transaction
	 * begin attempt failed.
	 * 
	 * @param txObject
	 *            the current transaction
	 */
	protected void closeEntityManagerAfterFailedBegin(
			ScTransactionObject txObject) {
		if (txObject.isNewEntityManagerHolder()) {
			ScEntityManager aemo = ((ScEntityManagerHolder) txObject
					.getScEntityManagerHolder()).getScEntityManager();
			EntityManager em = aemo.getJpaEntityManager();
			Connection conn = null;
			try {
				conn = aemo.getConnection();
				if (em.getTransaction().isActive()) {
					em.getTransaction().rollback();
				}
			} catch (Throwable ex) {
				logger.debug(
						"Could not rollback EntityManager after failed transaction begin",
						ex);
			} finally {
				EntityManagerFactoryUtils.closeEntityManager(em);
				if (conn != null)
					try {
						conn.close();
					} catch (SQLException e) {
						logger.debug(
								"Could not close connection after failed transaction begin",
								e);
					}
			}
		}
	}

	/**
	 * JPA transaction object, representing a EntityManagerHolder. Used as
	 * transaction object by JpaTransactionManager.
	 */
	private static class ScTransactionObject extends ScTransactionObjectSupport {

		private ScEntityManagerHolder entityHolder;

		private boolean newHolder;

		private Object transactionData;

		public void setScEntityManagerHolder(
				ScEntityManagerHolder entityHolder, boolean newHolder) {
			this.entityHolder = entityHolder;
			this.newHolder = newHolder;
		}

		public ScEntityManagerHolder getScEntityManagerHolder() {
			return this.entityHolder;
		}

		public boolean isNewEntityManagerHolder() {
			return this.newHolder;
		}

		public boolean hasTransaction() {
			return (this.entityHolder != null && this.entityHolder
					.isTransactionActive());
		}

		public void setTransactionData(Object transactionData) {
			this.transactionData = transactionData;
			this.entityHolder.setTransactionActive(true);
			if (transactionData instanceof SavepointManager) {
				this.entityHolder
						.setSavepointManager((SavepointManager) transactionData);
			}
		}

		public Object getTransactionData() {
			return this.transactionData;
		}

		public void setRollbackOnly() {
			EntityTransaction tx = this.entityHolder.getScEntityManager()
					.getTransaction();
			if (tx.isActive()) {
				tx.setRollbackOnly();
			}
		}

		public boolean isRollbackOnly() {
			EntityTransaction tx = this.entityHolder.getScEntityManager()
					.getTransaction();
			return tx.getRollbackOnly();
		}

		public Object createSavepoint() throws TransactionException {
			return getSavepointManager().createSavepoint();
		}

		public void rollbackToSavepoint(Object savepoint)
				throws TransactionException {
			getSavepointManager().rollbackToSavepoint(savepoint);
		}

		public void releaseSavepoint(Object savepoint)
				throws TransactionException {
			getSavepointManager().releaseSavepoint(savepoint);
		}

		private SavepointManager getSavepointManager() {
			if (!isSavepointAllowed()) {
				throw new NestedTransactionNotSupportedException(
						"Transaction manager does not allow nested transactions");
			}
			SavepointManager savepointManager = getScEntityManagerHolder()
					.getSavepointManager();
			if (savepointManager == null) {
				throw new NestedTransactionNotSupportedException(
						"JpaDialect does not support savepoints - check your JPA provider's capabilities");
			}
			return savepointManager;
		}
	}

	/**
	 * @return the ScEntityManager
	 */
	public ScEntityManager getScEntityManager() {
		if (ScEntityManager == null) {
			ScEntityManager = createEntityManagerForTransaction();
		}
		return ScEntityManager;
	}

	/**
	 * @param ScEntityManager
	 *            the ScEntityManager to set
	 */
	public void setScEntityManager(ScEntityManager ScEntityManager) {
		this.ScEntityManager = ScEntityManager;
	}

	/**
	 * Holder for suspended resources. Used internally by <code>doSuspend</code>
	 * and <code>doResume</code>.
	 */
	private static class SuspendedResourcesHolder {

		private final ScEntityManagerHolder entityManagerHolder;

		private SuspendedResourcesHolder(ScEntityManagerHolder emHolder) {
			this.entityManagerHolder = emHolder;
		}

		private ScEntityManagerHolder getScEntityManagerHolder() {
			return this.entityManagerHolder;
		}
	}

}
