package pl.slawas.scjdbc4jpa.dialect;

import java.sql.Connection;
import java.sql.SQLException;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.LockModeType;
import javax.persistence.Query;
import javax.sql.DataSource;

import org.hibernate.cfg.Configuration;
import org.hibernate.engine.Mapping;

import pl.slawas.scjdbc4jpa.core.ScEntityManager;
import pl.slawas.scjdbc4jpa.jpa.mapping.Table;

public abstract class DialectSupport implements ScEntityManager {

	private EntityManager jpaEntityManager;

	protected int batchSize;

	protected int batchCommitSize;

	protected DataSource dataSource;

	protected Configuration configuration;

	protected Mapping mapping;

	protected DialectSupport(EntityManager hibernateEntityManager, int batchSize,
			int batchCommitSize, DataSource dataSource,
			Configuration configuration, Mapping mapping) {

		this.jpaEntityManager = hibernateEntityManager;
		this.batchSize = batchSize;
		this.batchCommitSize = batchCommitSize;
		this.dataSource = dataSource;
		this.configuration = configuration;
		this.mapping = mapping;
	}

	public int getBatchCommitSize() {
		return this.batchCommitSize;
	}

	public int getBatchSize() {
		return this.batchSize;
	}

	public Configuration getConfiguration() {
		return this.configuration;
	}

	public Connection getConnection() throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}

	public EntityManager getJpaEntityManager() {
		return this.jpaEntityManager;
	}

	public Mapping getMapping() {
		return this.mapping;
	}

	public boolean isAllowedConfiguration() {
		return (this.configuration != null);
	}

	public boolean isAllowedMapping() {
		return (this.mapping != null);
	}

	public void release() {
		// TODO Auto-generated method stub

	}

	public void setTable(Table table) {
		// TODO Auto-generated method stub

	}

	public void clear() {
		this.jpaEntityManager.clear();

	}

	public void close() {
		this.jpaEntityManager.close();

	}

	public boolean contains(Object arg0) {
		return this.jpaEntityManager.contains(arg0);
	}

	public Query createNamedQuery(String arg0) {
		return this.jpaEntityManager.createNamedQuery(arg0);
	}

	public Query createNativeQuery(String arg0) {
		return this.jpaEntityManager.createNativeQuery(arg0);
	}

	@SuppressWarnings("rawtypes")
	public Query createNativeQuery(String arg0, Class arg1) {
		return this.jpaEntityManager.createNativeQuery(arg0, arg1);
	}

	public Query createNativeQuery(String arg0, String arg1) {
		return this.jpaEntityManager.createNativeQuery(arg0, arg1);
	}

	public Query createQuery(String arg0) {
		return this.jpaEntityManager.createQuery(arg0);
	}

	public <T> T find(Class<T> arg0, Object arg1) {
		return this.jpaEntityManager.find(arg0, arg1);
	}

	public void flush() {
		this.jpaEntityManager.flush();
	}

	public Object getDelegate() {
		return this.jpaEntityManager.getDelegate();
	}

	public FlushModeType getFlushMode() {
		return this.jpaEntityManager.getFlushMode();
	}

	public <T> T getReference(Class<T> arg0, Object arg1) {
		return this.jpaEntityManager.getReference(arg0, arg1);
	}

	public EntityTransaction getTransaction() {
		return this.jpaEntityManager.getTransaction();
	}

	public boolean isOpen() {
		return this.jpaEntityManager.isOpen();
	}

	public void joinTransaction() {
		if (!this.jpaEntityManager.getTransaction().isActive()) {
			this.jpaEntityManager.getTransaction().begin();
		}
	}

	public void lock(Object arg0, LockModeType arg1) {
		this.jpaEntityManager.lock(arg0, arg1);
	}

	public <T> T merge(T arg0) {
		return this.jpaEntityManager.merge(arg0);
	}

	public void persist(Object arg0) {
		this.jpaEntityManager.persist(arg0);

	}

	public void refresh(Object arg0) {
		this.jpaEntityManager.refresh(arg0);
	}

	public void remove(Object arg0) {
		if (!this.jpaEntityManager.contains(arg0)) {
			this.jpaEntityManager.refresh(arg0);
		}
		this.jpaEntityManager.remove(arg0);
	}

	public void setFlushMode(FlushModeType arg0) {
		this.jpaEntityManager.setFlushMode(arg0);
	}

	/**
	 * @return the dialect
	 */
	public Dialect getDialect() {
		if (this.getClass().getAnnotation(DialectAnnotation.class) != null) {
			return this.getClass().getAnnotation(DialectAnnotation.class)
					.value();
		}
		return null;
	}

}
