package pl.slawas.scjdbc4jpa.jdbc;

import java.lang.reflect.InvocationTargetException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Hashtable;

import pl.slawas.paging.Page;
import pl.slawas.paging.PagingParams;
import pl.slawas.scjdbc4jpa.config.LoadJdbcConfiguration;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCTableDesc;
import pl.slawas.scjdbc4jpa.jdbc.sql.functions._IFunction;

class DumpRowMapper extends ResultRowMapper {

	private static final long serialVersionUID = 6878543440362484173L;

	/**
	 * Konstruktor za pomocą którego pobierane są wiersze jakie zostaną
	 * znalezione za pomocą zapytania SQL z przedziału, gdzie podstwą zawarcia
	 * wiersza na liście jest jego numer. Wiersze są numerowane począwszy od
	 * wartości {@link #FIRST_ROW_POSITION}.
	 * <p>
	 * <font color="#aa0000">Uwaga!</font> Liczba wierszy wyniku
	 * <b>NIE</b>podlega restrykcjom związanym z maksymalną liczbą wierszy w
	 * wyniku {@link LoadJdbcConfiguration#RESULT_MAX_PAGES}.
	 * </p>
	 * 
	 * Gdy użyjemy tego konstruktora zmieni się znaczenie i działanie metod:
	 * <ol>
	 * <li>
	 * {@link #getRows()}: metoda nie bedzie zwracać wszystkich wierszy tylko
	 * listę wierszy o numerach większych lub równych parametrowi
	 * 'startPosition' i mniejszych lub równych parametrowi 'endPosition'</li>
	 * <li>
	 * {@link #getRows(Integer, Integer)}: dla tej metody początkowy zbór
	 * wyników bedzie już wcześniej ograniczony do listy wierszy o numerach
	 * większych lub równych parametrowi 'startPosition' i mniejszych lub
	 * równych parametrowi 'endPosition'</li>
	 * </ol>
	 * 
	 * @param tableDescription
	 *            opis tabeli
	 * @param rs
	 *            wynik zapytania
	 * @param createEntityObject
	 *            czy ma być tworzony obiekt encji (jeżeli {@code true}, to
	 *            obiekt encji bedzie utworzony i dostępny w obiekcie
	 *            {@link ResultRow}). Obiekt może nie być tworzony w przypadku
	 *            kiesdy zapytanie składa się tylko z funkcji
	 *            {@link SelectQuery#setUseFunctionsOnly(boolean)}
	 * @param startPosition
	 *            początkowy numer wiersza. Jeżeli podamy wartość {@code null},
	 *            lista bedzie tworzona od pierwszego wiersza.
	 * @param endPosition
	 *            końcowy numer wiersza. Jeżeli podamy wartość {@code null},
	 *            lista bedzie tworzona od ostatniego wiersza.
	 * @param pagingParams
	 *            Parametry stronicowania. Jeżeli zostanie podany jako wartość
	 *            {@code null}, to wtedy zostanie przyjęty domyślny rozmiar
	 *            strony zdefiniowany w stałej {@link Page#DEFAULT_PAGE_SIZE}.
	 *            <font color="#009900">Maksymalna liczba stron
	 *            {@link LoadJdbcConfiguration#RESULT_MAX_PAGES} dla tego
	 *            obiektu nie ma znaczenia</font>
	 * @param functions
	 *            tablica użytych w zapytaniu funkcji. Wyniki użytuch funkcji
	 *            dostepne będą w obiekcie {@link ResultRow}) (za pomocą metod
	 *            {@link ResultRow#getColumnsKeys()} oraz
	 *            {@link ResultRow#getColumnValue(String)}).
	 * @param columnsToUse
	 *            niedomyślna lista kolumn użyta w zapytaniu. Niedomyślna
	 *            oznacza, że w zapytaniu biorą new wszystkie kolumny
	 *            odpowiedające polom encji.
	 * 
	 * @throws SQLException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws NoSuchMethodException
	 * @throws SecurityException
	 * @throws InvocationTargetException
	 * @throws IllegalArgumentException
	 */
	public DumpRowMapper(JDBCTableDesc tableDescription, ResultSet rs,
			boolean createEntityObject, Integer startPosition,
			Integer endPosition, PagingParams pagingParams,
			Hashtable<String, _IFunction> functions, String[] columnsToUse)
			throws SQLException, InstantiationException,
			IllegalAccessException, SecurityException, NoSuchMethodException,
			IllegalArgumentException, InvocationTargetException {
		super(tableDescription, rs, createEntityObject, startPosition,
				endPosition, pagingParams, functions, columnsToUse,
				ResultType.UNLIMITED);
	}

}
