package pl.slawas.scjdbc4jpa.jdbc;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.persistence.EntityManager;
import javax.persistence.FlushModeType;
import javax.sql.DataSource;

import org.apache.commons.lang.StringUtils;
import org.hibernate.cfg.Configuration;
import org.hibernate.engine.Mapping;

import pl.slawas.paging.PagingParams;
import pl.slawas.scjdbc4jpa.config.JdbcConfig;
import pl.slawas.scjdbc4jpa.config.JdbcConfigEntityMetadata;
import pl.slawas.scjdbc4jpa.config.LoadJdbcConfiguration;
import pl.slawas.scjdbc4jpa.core.ScEntityManagerFactorySupport;
import pl.slawas.scjdbc4jpa.dialect.Dialect;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCKeyDesc;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCTableDesc;
import pl.slawas.scjdbc4jpa.jdbc.cache.CacheUsage;
import pl.slawas.scjdbc4jpa.jdbc.exceptions.CreateSQLQueryException;
import pl.slawas.scjdbc4jpa.jdbc.exceptions.IdIsNotEmbeddableException;
import pl.slawas.scjdbc4jpa.jdbc.exceptions.JDBCEntityManagerException;
import pl.slawas.scjdbc4jpa.jdbc.exceptions.JDBCEntityParserException;
import pl.slawas.scjdbc4jpa.jdbc.exceptions.ObjectIsNotEntityException;
import pl.slawas.scjdbc4jpa.jdbc.utils.JDBCEntityParserLevel;
import pl.slawas.scjdbc4jpa.jpa._JpaEntityManagerFactory;
import pl.slawas.scjdbc4jpa.security.SecurePasswordIdentity;

/**
 * 
 * JDBCEntityManagerFactory
 * 
 * @see JDBCEntityManager
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.2 $
 * 
 */
public class JDBCEntityManagerFactory extends ScEntityManagerFactorySupport
		implements _JpaEntityManagerFactory {

	private static final long serialVersionUID = -5652152295214814881L;

	/* OBIEKTY PRYWATNE wykonawcze - start */
	private boolean isOpen = true;

	/**
	 * sekwencja generowanych instancji {@link JDBCEntityManager} potrzebna do
	 * budowania unikalnych regionów kesza pierwszego poziomu
	 */
	private JDBCEntityManagerSequence entityManagerSequence = JDBCEntityManagerSequence
			.getInstance();

	/** Metadane encji. Kluczem jest nazwa klasy encji */
	private ConcurrentMap<String, JDBCEntityMetaData> metaData;

	/* OBIEKTY PRYWATNE wykonawcze - koniec */

	/* PARAMETRY - start */
	private String configUrl;

	private JdbcConfig config;

	private DataSource dataSource;

	private boolean testingConfig = false;

	private Dialect dbDialect;

	/**
	 * rozmiar strony wyników
	 */
	private PagingParams pagingParams;

	private FlushModeType flushModeType = FlushModeType.COMMIT;

	/* PARAMETRY - koniec */
	/**
	 * Dana fabryka nie wspiera konfiguracji Hibrnate.
	 */
	@Deprecated
	public Configuration getConfiguration() {
		throw new UnsupportedOperationException(
				"Dana fabryka nie wspiera konfiguracji Hibrnate.");
	}

	public DataSource getDataSource() {
		return dataSource;
	}

	/**
	 * Dana fabryka nie wspiera mapowania Hibrnate.
	 */
	@Deprecated
	public Mapping getMapping() {
		throw new UnsupportedOperationException(
				"Dana fabryka nie wspiera mapowania Hibrnate.");
	}

	public void close() {
		isOpen = false;
	}

	public synchronized EntityManager createEntityManager() {
		JDBCEntityManager newEm = new JDBCEntityManager(
				this.pagingParams.copy(), this);
		/* ustawienie niedomyślnych parametrów */
		newEm.setBatchSize(this.getBatchSize());
		newEm.setBatchCommitSize(this.getBatchCommitSize());
		newEm.setFlushMode(this.flushModeType);
		return newEm;
	}

	@SuppressWarnings({ "rawtypes" })
	public synchronized EntityManager createEntityManager(Map map) {
		JDBCEntityParserLevel level = JDBCEntityParserLevel.THIRD;
		if (map.containsKey("level")) {
			level = JDBCEntityParserLevel.valueOf((String) map.get("level"));
		}
		String dbUser = null;
		if (map.containsKey("dbUser")) {
			dbUser = (String) map.get("dbUser");
		}
		String dbPassword = null;
		if (map.containsKey("dbPassword")) {
			dbPassword = (String) map.get("dbPassword");
		}
		PagingParams pagingParams = this.pagingParams.copy();
		if (map.containsKey("pageResultSize")) {
			pagingParams = LoadJdbcConfiguration.getPagingParams(Integer
					.parseInt((String) map.get("pageResultSize")));
		}
		if (pagingParams != null && map.containsKey("maxPages")) {
			int maxPages = Integer.parseInt((String) map.get("maxPages"));
			pagingParams.setMaxCount(maxPages * pagingParams.getPageSize());
		}

		JDBCEntityManager newEm = new JDBCEntityManager(pagingParams, dbUser,
				dbPassword, this, level);

		/* ustawienie niedomyślnych parametrów */
		int batchSize = this.getBatchSize();
		if (map.containsKey("batchSize")) {
			batchSize = Integer.parseInt((String) map.get("batchSize"));
		}
		newEm.setBatchSize(batchSize);
		int batchCommitSize = this.getBatchCommitSize();
		if (map.containsKey("batchCommitSize")) {
			batchCommitSize = Integer.parseInt((String) map
					.get("batchCommitSize"));
		}
		newEm.setBatchCommitSize(batchCommitSize);
		FlushModeType flushModeType = this.flushModeType;
		if (map.containsKey("flushModeType")) {
			flushModeType = FlushModeType.valueOf((String) map
					.get("flushModeType"));
		}
		newEm.setFlushMode(flushModeType);
		return newEm;
	}

	public boolean isOpen() {
		return isOpen;
	}

	public _JpaEntityManagerFactory getObject() throws Exception {
		return this;
	}

	@SuppressWarnings("rawtypes")
	public Class getObjectType() {
		return _JpaEntityManagerFactory.class;
	}

	public boolean isSingleton() {
		return true;
	}

	/**
	 * Odczytuje konfiguracje z pliku konfiguracyjnego. Konfiguracja bedzie
	 * odczytana z domyślnej lokalizacji albo z pliku o zdefiniowanym parametrze
	 * {@link #configUrl} (można go ustawić z poziomu SPRING-a).
	 * 
	 * Parametry źródła danych (namiary na nie) pozwalam nadpisywać z poziomu
	 * konfiguracji SPRING-a. Są to parametry:
	 * <ul>
	 * <li> {@link ScEntityManagerFactorySupport#jnpDisableDiscovery}
	 * <li> {@link ScEntityManagerFactorySupport#providerUrl}
	 * <li> {@link ScEntityManagerFactorySupport#urlPkgPrefixes}
	 * <li> {@link ScEntityManagerFactorySupport#initialContextFactory}
	 * <li> {@link ScEntityManagerFactorySupport#dsJndiName}
	 * <li>
	 * {@link ScEntityManagerFactorySupport#setDataSource(DataSource)}
	 * <li> {@link ScEntityManagerFactorySupport#setDialectName(String)}
	 * <li> {@link ScEntityManagerFactorySupport#setDbUser(String)}
	 * <li>
	 * {@link ScEntityManagerFactorySupport#setEncodedDbPassword(String)}
	 * <li> {@link ScEntityManagerFactorySupport#setDbPassword(String)}
	 * </ul>
	 * Pozostałe parametry, nawet jeśli zostaną zdefiniowane w konfiguracji
	 * SPRING-a, są zaczytywane z obiektu konfiguracji {@link JdbcConfig}.
	 * 
	 */
	public void afterPropertiesSet() throws Exception {
		if (org.apache.commons.lang.StringUtils.isBlank(configUrl)) {
			this.configUrl = JdbcConfig.DEFAULT_CONFIG_URL;
		}
		this.config = (new LoadJdbcConfiguration(this.configUrl, true))
				.makeIndexConfig();

		/** wpierw ustalam parametry źródła danych - START */
		// jnpDisableDiscovery;
		if (StringUtils.isBlank(this.jnpDisableDiscovery)
				&& StringUtils.isNotBlank(config.getJnpDisableDiscovery())) {
			this.setJnpDisableDiscovery(config.getJnpDisableDiscovery());
		}
		// providerUrl;
		if (StringUtils.isBlank(this.providerUrl)
				&& StringUtils.isNotBlank(config.getProviderUrl())) {
			this.setProviderUrl(config.getProviderUrl());
		}
		// urlPkgPrefixes;
		if (StringUtils.isBlank(this.urlPkgPrefixes)
				&& StringUtils.isNotBlank(config.getUrlPkgPrefixes())) {
			this.setUrlPkgPrefixes(config.getUrlPkgPrefixes());
		}
		// initialContextFactory;
		if (StringUtils.isBlank(this.initialContextFactory)
				&& StringUtils.isNotBlank(config.getInitialContextFactory())) {
			this.setInitialContextFactory(config.getInitialContextFactory());
		}
		// dsJndiName = DetaultJndiName;
		if (StringUtils.isBlank(this.dsJndiName)
				&& StringUtils.isNotBlank(config.getJndiName())) {
			this.dsJndiName = config.getJndiName();
		}
		// dialekt bazodanowy;
		if (this.dbDialect == null && config.getDbDialect() != null) {
			this.setDataBaseDialect(config.getDbDialect());
		}
		if (StringUtils.isBlank(getDbUser())
				&& StringUtils.isNotBlank(config.getDbUser())) {
			this.setDbUser(config.getDbUser());
		}
		if (StringUtils.isBlank(getDbPassword())
				&& StringUtils.isNotBlank(config.getDbPassword())) {
			this.setEncodedDbPassword(SecurePasswordIdentity.encode(config
					.getDbPassword()));
		}

		/** wpierw ustalam parametry źródła danych - KONIEC */

		/** pozostale parametry konfiguracyjne - START */
		// parametry przetwarzania batchowego
		if (config.getBatchCommitSize() != null) {
			this.setBatchCommitSize(config.getBatchCommitSize());
		}
		if (config.getBatchSize() != null) {
			this.setBatchSize(config.getBatchSize());
		}

		if (this.pagingParams == null) {
			this.pagingParams = LoadJdbcConfiguration.getPagingParams(config
					.getPageResultSize());
			if (config.getMaxPages() != null) {
				pagingParams.setMaxCount(config.getMaxPages()
						* pagingParams.getPageSize());
			}
		}

		if (config.getFlushModeType() != null) {
			this.flushModeType = config.getFlushModeType();
		}
		if (config.getDeclaredEntities() != null
				&& !config.getDeclaredEntities().isEmpty()) {
			createMetaData();
		}
		/** pozostale parametry konfiguracyjne - KONIEC */

		if (!this.testingConfig)
			super.afterPropertiesSet();
	}

	/**
	 * Metoda pomocnicza do ustawiania dialektu. Jako, że został odzedziczony
	 * parametr {@link ScEntityManagerFactorySupport#dialectName}, trzeba zadbać
	 * by był on również poprawnie ustawiony.
	 * 
	 * @param dataBaseDialect
	 */
	private void setDataBaseDialect(Dialect dataBaseDialect) {
		this.dbDialect = dataBaseDialect;
		this.dialectName = this.dbDialect.getEmClass().getName();
	}

	/**
	 * Metoda tworząca metadane na podstawie konfiguracji.
	 * 
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws Exception
	 */
	private void createMetaData() throws ClassNotFoundException,
			NoSuchMethodException, Exception {
		metaData = new ConcurrentHashMap<String, JDBCEntityMetaData>();
		Hashtable<String, JdbcConfigEntityMetadata> tmpListOfDeclaredEntities = new Hashtable<String, JdbcConfigEntityMetadata>();
		for (JdbcConfigEntityMetadata md : config.getDeclaredEntities()) {
			tmpListOfDeclaredEntities.put(md.getEntityName(), md);
		}
		for (JdbcConfigEntityMetadata md : config.getDeclaredEntities()) {
			createMetaData(md, tmpListOfDeclaredEntities);
		}
	}

	/**
	 * Metoda dająca możliwość dodatkowe zarejestrowanie encji.
	 * <p>
	 * <b>Uwaga!</b> Dodatkowe rejestrowanie encji można dokonać dopiero po
	 * inicjalizacji fabryki, czyli po wywołaniu metody
	 * {@link #afterPropertiesSet()}
	 * </p>
	 * 
	 * @param md
	 *            metadane nowej encji
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws Exception
	 */
	public void registerMetaData(JdbcConfigEntityMetadata md)
			throws ClassNotFoundException, NoSuchMethodException, Exception {
		String entity = md.getEntityName();
		if (metaData == null) {
			metaData = new ConcurrentHashMap<String, JDBCEntityMetaData>();
		} else if (metaData.get(entity) != null) {
			return;
		}

		Hashtable<String, JdbcConfigEntityMetadata> declaredEntities = new Hashtable<String, JdbcConfigEntityMetadata>();
		if (config.getDeclaredEntities() != null) {
			for (JdbcConfigEntityMetadata currentMD : config
					.getDeclaredEntities()) {
				declaredEntities.put(currentMD.getEntityName(), currentMD);
			}
		}
		createMetaData(md, declaredEntities);

	}

	/**
	 * @param md
	 * @param tmpListOfDeclaredEntities
	 * 
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws Exception
	 */
	private void createMetaData(
			JdbcConfigEntityMetadata md,
			Hashtable<String, JdbcConfigEntityMetadata> tmpListOfDeclaredEntities)
			throws ClassNotFoundException, NoSuchMethodException, Exception {

		String entity = md.getEntityName();
		if (metaData.get(entity) != null)
			return;

		String cacheRegionName = md.getCacheRegionName();
		CacheUsage cacheUsage = md.getCacheUsage();
		CacheUsage queryCacheUsage = md.getQueryCacheUsage();
		try {
			Class<?> entityClass = Thread.currentThread()
					.getContextClassLoader().loadClass(entity);
			logger.info("Znaleziono encje: {}", entityClass.getName());
			JDBCEntityMetaData jDBCEntityMetaData = new JDBCEntityMetaData(
					this.dbDialect, entityClass, this.pagingParams.copy(),
					cacheUsage, cacheRegionName, queryCacheUsage);
			logger.debug("Metadane encji {}: \n{}",
					new Object[] { entityClass.getSimpleName(),
							jDBCEntityMetaData.toString() });
			metaData.put(entity, jDBCEntityMetaData);
			/*
			 * sprawdzam czy tabela ma klucze obce i od razu tez tworze dla nich
			 * metadane.
			 */
			JDBCTableDesc tabDef = jDBCEntityMetaData
					.getTableDefinition(JDBCEntityParserLevel.THIRD);
			Enumeration<JDBCKeyDesc> fks = tabDef.getForeignKeys();
			while (fks.hasMoreElements()) {
				JDBCTableDesc fkTable = tabDef
						.getJoinedTable(fks.nextElement());
				JdbcConfigEntityMetadata declaredEntity = tmpListOfDeclaredEntities
						.get(fkTable.getEntityClass().getName());
				if (declaredEntity != null) {
					createMetaData(declaredEntity, tmpListOfDeclaredEntities);
				} else {
					createMetaData(
							new JdbcConfigEntityMetadata(fkTable
									.getEntityClass().getName(), cacheUsage,
									fkTable.getCacheRegionName(),
									queryCacheUsage), tmpListOfDeclaredEntities);
				}
			}

		} catch (ObjectIsNotEntityException e) {
			throw new Exception(e);
		} catch (JDBCEntityParserException e) {
			throw new Exception(e);
		} catch (IdIsNotEmbeddableException e) {
			throw new Exception(e);
		} catch (CreateSQLQueryException e) {
			throw new Exception(e);
		}
	}

	public void destroy() throws Exception {
		logger.info("Zamykam JDBCEntityManagerFactory...");
		this.close();
	}

	@Override
	protected void setterOfDataSourceMethodImplementation(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	/**
	 * @return następna wartość sekwencji otwartych managarów encji
	 */
	long getEntityManagerSequenceNextValue() {
		return this.entityManagerSequence.getNextValue();
	}

	/**
	 * @return the config
	 */
	public JdbcConfig getConfig() {
		return config;
	}

	/**
	 * @param configUrl
	 *            ustawienie ścieżki parametrów kontiguracyjnych
	 */
	public void setConfigUrl(String configUrl) {
		this.configUrl = configUrl;
	}

	public JDBCEntityMetaData getMetaData(String entityClassName) {
		return this.metaData.get(entityClassName);
	}

	/**
	 * @return the dbDialect
	 */
	public Dialect getDbDialect() {
		return this.dbDialect;
	}

	@Override
	public void setDialectName(String dialectName) {
		try {
			Class<?> emClass = this.getClass().getClassLoader()
					.loadClass(dialectName);
			this.dbDialect = Dialect.getDialect(emClass);
			super.setDialectName(dialectName);
		} catch (ClassNotFoundException e) {
			throw new JDBCEntityManagerException("Blad ustawienia dialektu", e);
		}
	}

	/**
	 * @param testingConfig
	 *            the testingConfig to set
	 */
	public void setTestingConfig(boolean testingConfig) {
		this.testingConfig = testingConfig;
	}

	/**
	 * @param pagingParams
	 *            the pagingParams to set
	 */
	public void setPagingParams(PagingParams pagingParams) {
		this.pagingParams = pagingParams;
	}

	/**
	 * @param flushModeType
	 *            the flushModeType to set
	 */
	public void setFlushModeType(FlushModeType flushModeType) {
		this.flushModeType = flushModeType;
	}

}
