package pl.slawas.scjdbc4jpa.jdbc;

import java.util.Collection;

import pl.slawas.paging.PagingParams;
import pl.slawas.paging.ResultMessage;
import pl.slawas.paging.ResultSupport;
import pl.slawas.scjdbc4jpa.config.LoadJdbcConfiguration;

/**
 * 
 * QueryResult wynik zapytania SQL
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.5 $
 * 
 */
public class QueryResult extends ResultSupport<ResultRow> {

	private static final long serialVersionUID = 8586577008473662383L;

	private final SelectQuery.QueryCacheInfo queryCacheInfo;

	/**
	 * @param queryCacheInfo
	 *            informacje zwiazane z keszowaniem zapytania.
	 * @param startPosition
	 *            numer pierwszej pozycji strony (pozycja kursora w calym
	 *            rezultacie)
	 * @param endPosition
	 *            numer ostatniej pozycji strony (pozycja kursora w calym
	 *            rezultacie)
	 * @param firstRowPosition
	 *            pierwsza pozycja rezultatu (pozycja kursora w calym
	 *            rezultacie)
	 * @param resultSize
	 *            rozmiar rezultatu
	 * @param lastRowPosition
	 *            ostatnia pozycja rezultatu (pozycja kursora w calym
	 *            rezultacie)
	 * @param result
	 *            lista wyników rezultatu
	 * @param message
	 *            komunikat związany z rezultatem ({@link QueryResulMessage})
	 * @param pagingParams
	 *            Parametry stronicowania. nie mogą być puste ({@code null})
	 */
	QueryResult(SelectQuery.QueryCacheInfo queryCacheInfo, int startPosition,
			int endPosition, Integer firstRowPosition, int resultSize,
			Integer lastRowPosition, Collection<ResultRow> result,
			ResultMessage message, PagingParams pagingParams) {
		super(startPosition, endPosition, firstRowPosition, resultSize,
				lastRowPosition, result, message,
				LoadJdbcConfiguration.FIRST_ROW_POSITION, pagingParams
						.getMaxPages());
		this.queryCacheInfo = queryCacheInfo;
	}

	/**
	 * @return the queryCacheInfo
	 */
	public SelectQuery.QueryCacheInfo getQueryCacheInfo() {
		return queryCacheInfo;
	}

}
