package pl.slawas.scjdbc4jpa.jdbc;

import java.io.Serializable;
import java.util.Vector;

import pl.slawas.scjdbc4jpa.jdbc.sort.SortCondition;

/**
 * 
 * SortClause klauzula sortowania wyników zapytania SQL
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.3 $
 * 
 */
public class SortClause implements _IClause, Serializable {

	private static final long serialVersionUID = -7307968594304256779L;

	private final Vector<SortCondition> sortConditionsList;

	private final SortCondition[] sortConditionsArray;

	public SortClause(Vector<SortCondition> sortConditions) {
		if (sortConditions == null || sortConditions.isEmpty())
			throw new IllegalArgumentException(
					"Lista warunkow sortowania nie moze byc pusta");

		this.sortConditionsList = sortConditions;
		this.sortConditionsArray = new SortCondition[this.sortConditionsList
				.size()];
		int i = 0;
		for (SortCondition cond : this.sortConditionsList) {
			this.sortConditionsArray[i] = cond;
			i++;
		}
	}

	public SortClause(SortCondition[] sortConditions) {
		if (sortConditions == null || sortConditions.length == 0)
			throw new IllegalArgumentException(
					"Lista warunkow sortowania nie moze byc pusta");

		this.sortConditionsArray = sortConditions;
		this.sortConditionsList = new Vector<SortCondition>(
				this.sortConditionsArray.length);
		for (SortCondition cond : this.sortConditionsArray) {
			this.sortConditionsList.add(cond);
		}
	}

	/**
	 * @return the sortConditionsList
	 */
	public Vector<SortCondition> getSortConditionsList() {
		return sortConditionsList;
	}

	/**
	 * @return the sortConditionsArray
	 */
	public SortCondition[] getSortConditionsArray() {
		return sortConditionsArray;
	}

	/**
	 * Metoda pobierania (budowania) klauzuli sortowania
	 * 
	 * @return klauzula sortowania
	 */
	public String getClause() {
		StringBuffer out = new StringBuffer("ORDER BY");
		for (int i = 0; i < this.sortConditionsArray.length; i++) {
			out.append((0 == i ? " " : ",")
					+ this.sortConditionsArray[i].getClause());
		}
		return out.toString();
	}

}
