package pl.slawas.scjdbc4jpa.jdbc.beans;

import java.io.Serializable;
import java.util.Comparator;
import java.util.Hashtable;

/**
 * 
 * JdbcKeyDesc definicja klucza
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.2 $
 * 
 */
public class JDBCKeyDesc implements Serializable {

	private static final long serialVersionUID = -554945885612689796L;

	private final String name;

	private final JDBCKeyType type;

	private final Class<?> keyType;

	/**
	 * Lista kolumn. Pierwszy parametr topozycja pola w kluczu, drugi to opis
	 * kolumny
	 */
	private final Hashtable<Integer, JDBCColumnDesc> columns = new Hashtable<Integer, JDBCColumnDesc>();

	/**
	 * 
	 * @param name
	 *            nazwa klucza
	 * @param type
	 *            typ klucza
	 */
	public JDBCKeyDesc(Class<?> keyType, String name, JDBCKeyType type) {
		super();
		this.keyType = keyType;
		this.name = name;
		this.type = type;
	}

	/**
	 * @return lista kolumn
	 */
	public Hashtable<Integer, JDBCColumnDesc> getColumns() {
		return columns;
	}

	/**
	 * @return macierz kolumn (kolumny w kolejności w/g pozycji)
	 */
	public JDBCColumnDesc[] getColumnsArray() {
		JDBCColumnDesc[] columnsArray = new JDBCColumnDesc[columns.size()];
		for (int i = 0; i < columnsArray.length; i++) {
			columnsArray[i] = columns.get(i);
		}
		return columnsArray;
	}

	/**
	 * Zwraca opis kolumny na danej pozycji
	 * 
	 * @param position
	 *            pozycja kolumny w kluczu (zaczynamy od {@code 0})
	 * @return opis kolumny
	 */
	public JDBCColumnDesc getColumn(Integer position) {
		return columns.get(position);
	}

	/**
	 * Dodawanie kolumny do klucza
	 * 
	 * @param position
	 *            pozycja kolumny w kluczu (zaczynamy od {@code 0})
	 * @param column
	 *            definicja kolumny
	 * @return tworzony klucz
	 */
	public JDBCKeyDesc addColumn(Integer position, JDBCColumnDesc column) {
		columns.put(position, column);
		return this;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the type
	 */
	public JDBCKeyType getType() {
		return type;
	}

	/**
	 * @return the keyType
	 */
	public Class<?> getKeyType() {
		return keyType;
	}

	@Override
	public String toString() {
		return "JDBCKeyDesc [\n\tkeyType=" + keyType + "\n\tname=" + name
				+ "\n\ttype=" + type + "]";
	}

	public static class SortByName implements Comparator<JDBCKeyDesc> {

		public int compare(JDBCKeyDesc o1, JDBCKeyDesc o2) {
			return o1.getName().toLowerCase()
					.compareTo(o2.getName().toLowerCase());
		}
	}

}
