package pl.slawas.scjdbc4jpa.jdbc.cache;

import pl.slawas.scjdbc4jpa.config.JpaConfig;
import pl.slawas.scjdbc4jpa.config.JpaProperties;

public enum CacheProviderEnum {

	EhCache, none;

	public _IObjectCacheProvider getProvider() {
		switch (this) {
		case EhCache:
			return EhCacheProvider.getInstance();
		case none:
			return null;
		default:
			throw new UnsupportedOperationException();
		}
	}

	public boolean isAllowed() {
		switch (this) {
		case EhCache:
			return true;
		case none:
		default:
			return false;
		}
	}

	/**
	 * Nazwa provider-a kesza atrybutow, domyslna wartość
	 * {@link CacheProviderEnum#none#toString()}
	 */
	public static final String CACHE_PROVIDER = (JpaConfig.getInstance().get(
			JpaProperties.CACHE_PROVIDER) == null ? CacheProviderEnum.none
			.toString() : JpaConfig.getInstance().get(
			JpaProperties.CACHE_PROVIDER));

	/**
	 * Czas przechowywanie elementow w cache w [s]. domyslna wartośc jest
	 * zdefiniowana w pliku konfiguracyjnym parametrem
	 * {@link #CACHE_TIME_TO_LIVE}, jednak gdy nie jest on zdefiniowany
	 * przyjmuje on wartosc {@code 3600} (1h).
	 */
	public static long cacheTimeToLive = (JpaConfig.getInstance().get(
			JpaProperties.CACHE_TIME_TO_LIVE) == null ? 3600000 : Integer
			.valueOf(JpaConfig.getInstance().get(
					JpaProperties.CACHE_TIME_TO_LIVE)));

	/**
	 * Informacje o tym, czy podczas używania kesz-a, ma zostać użyty jeden
	 * domyślny region
	 * 
	 * @see {@link JpaConfig#CACHE_USE_DEFAULT_REGION}
	 */
	public static final Boolean CACHE_USE_DEFAULT_REGION = (JpaConfig
			.getInstance().get(JpaProperties.CACHE_USE_DEFAULT_REGION) == null ? false
			: Boolean.parseBoolean(JpaConfig.getInstance().get(
					JpaProperties.CACHE_USE_DEFAULT_REGION)));

}
