package pl.slawas.scjdbc4jpa.jdbc.cache;

import java.io.Serializable;
import java.util.Hashtable;

import net.sf.ehcache.CacheManager;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import pl.slawas.helpers.FileUtils;

/**
 * 
 * EhCacheProvider - utworzenie instancji CacheManager-a
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.3 $
 * 
 */
public class EhCacheProvider implements Serializable, _IObjectCacheProvider {

	private static final long serialVersionUID = -6327091572644983790L;

	public static String FILE_CONFIG_PATH = FileUtils.resolveName(
			EhCacheProvider.class, "ehcache.xml");

	private static final Logger logger = LoggerFactory
			.getLogger(EhCacheProvider.class);

	private static _IObjectCacheProvider instance;

	private static CacheManager manager;

	private Hashtable<String, _IObjectCache> caches = new Hashtable<String, _IObjectCache>();

	private EhCacheProvider() {
		if (CUSTOM_CONFIG_PATH != null) {
			logger.info("Laduje konfiguracje z pliku: " + CUSTOM_CONFIG_PATH);
			manager = new CacheManager(CUSTOM_CONFIG_PATH);
		} else {
			logger.info("Laduje domyslna konfiguracje.");
			manager = new CacheManager(
					EhCacheProvider.class.getResourceAsStream(FILE_CONFIG_PATH));
		}
		for (String key : getCacheNames()) {
			caches.put(key, new EhCache(manager.getCache(key)));
		}
	}

	public synchronized static _IObjectCacheProvider getInstance() {
		if (instance == null) {
			instance = new EhCacheProvider();
		}
		return instance;
	}

	public String[] getCacheNames() {
		return manager.getCacheNames();
	}

	public _IObjectCache getCache(String name) {
		_IObjectCache cache = caches.get(name);
		if (cache == null) {
			logger.debug("Tworze nowy kesz o nazwie '{}'", name);
			manager.addCache(name);
			cache = new EhCache(manager.getCache(name));
			caches.put(name, cache);
		}
		return cache;
	}

	public synchronized void close() {
		manager.shutdown();
		instance = null;
	}

	public void removeCache(String name) {
		_IObjectCache cache = caches.get(name);
		if (cache != null) {
			logger.debug("Usuwam kesz o nazwie '{}'", name);
			manager.removeCache(name);
			caches.remove(name);
		}
	}

}
