package pl.slawas.scjdbc4jpa.jdbc.cache;

import java.util.Map;

import pl.slawas.scjdbc4jpa.jdbc.exceptions.ObjectCacheErrorException;

import net.sf.ehcache.CacheManager;

public interface _IObjectCache {

	/**
	 * Gets a value of an element which matches the given key.
	 * 
	 * @param key
	 *            the key of the element to return.
	 * @return The value placed into the cache with an earlier put, or null if
	 *         not found or expired
	 * @throws ObjectCacheErrorException
	 */
	public abstract Object get(Object key) throws ObjectCacheErrorException;

	public abstract Object read(Object key) throws ObjectCacheErrorException;

	/**
	 * Puts an object into the cache.
	 * 
	 * @param key
	 *            a key
	 * @param value
	 *            a value
	 * @throws ObjectCacheErrorException
	 *             if the {@link CacheManager} is shutdown or another
	 *             {@link Exception} occurs.
	 */
	public abstract void update(Object key, Object value)
			throws ObjectCacheErrorException;

	/**
	 * Puts an object into the cache.
	 * 
	 * @param key
	 *            a key
	 * @param value
	 *            a value
	 * @throws ObjectCacheErrorException
	 *             if the {@link CacheManager} is shutdown or another
	 *             {@link Exception} occurs.
	 */
	public abstract void put(Object key, Object value)
			throws ObjectCacheErrorException;

	/**
	 * Removes the element which matches the key.
	 * <p/>
	 * If no element matches, nothing is removed and no Exception is thrown.
	 * 
	 * @param key
	 *            the key of the element to remove
	 * @throws ObjectCacheErrorException
	 */
	public abstract void remove(Object key) throws ObjectCacheErrorException;

	/**
	 * Remove all elements in the cache, but leave the cache in a useable state.
	 * 
	 * @throws ObjectCacheErrorException
	 */
	public abstract void clear() throws ObjectCacheErrorException;

	/**
	 * Remove the cache and make it unuseable.
	 * 
	 * @throws ObjectCacheErrorException
	 */
	public abstract void destroy() throws ObjectCacheErrorException;

	public abstract String getRegionName();

	/**
	 * Warning: This method can be very expensive to run. Allow approximately 1
	 * second per 1MB of entries. Running this method could create liveness
	 * problems because the object lock is held for a long period
	 * <p/>
	 * 
	 * @return the approximate size of memory ehcache is using for the
	 *         MemoryStore for this cache
	 */
	public abstract long getSizeInMemory();

	public abstract long getElementCountInMemory()
			throws ObjectCacheErrorException;

	public abstract long getElementCountOnDisk();

	@SuppressWarnings("rawtypes")
	public abstract Map toMap() throws ObjectCacheErrorException;

	public abstract String toString();

	public abstract _IObjectCacheStatistics getStatistics();

	public void setTimeToLiveSeconds(long timeToLiveSeconds);

	public long getTimeToLiveSeconds();

}