package pl.slawas.scjdbc4jpa.jdbc.cache.managers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCTableDesc;
import pl.slawas.scjdbc4jpa.jdbc.cache.CacheSharing;
import pl.slawas.scjdbc4jpa.jdbc.cache.CacheUsage;
import pl.slawas.scjdbc4jpa.jdbc.cache.ObjectWithCacheSupport;
import pl.slawas.scjdbc4jpa.jdbc.exceptions.ObjectCacheErrorException;

/**
 * 
 * QueryCacheManager zarządzjąca keszem zapytania dla danej encji i zapytań
 * danego typu.
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.2 $
 * 
 */
public class QueryCacheManager extends ObjectWithCacheSupport {

	private static final long serialVersionUID = 4899440999809248586L;

	final static private Logger logger = LoggerFactory
			.getLogger(QueryCacheManager.class);

	protected QueryCacheManager(CacheUsage cacheUsage, String regionName,
			CacheSharing cacheSharing) {
		super(cacheUsage, regionName, cacheSharing, false);
	}

	/**
	 * 
	 * @param cacheUsage
	 *            informacja o tym czy ma być używany kesz. Jako, że jest to
	 *            inicjalizacjia obiektu, to parametr powinien przyjmować jedną
	 *            z dwóch wartości: {@link CacheUsage#TO_USE} oraz
	 *            {@link CacheUsage#NONE}
	 * @param tableDefinition
	 *            definicja tabeli, po to by między innymi na jej podstawie
	 *            ustalić nazwę regionu dla kesz-a
	 * @param type
	 *            typ keszowanych zapytań, po to by między innymi na jej
	 *            podstawie ustalić nazwę regionu dla kesz-a
	 */
	public QueryCacheManager(CacheUsage cacheUsage,
			JDBCTableDesc tableDefinition, QueryCacheType type) {
		this(cacheUsage, createQueryRegionName(tableDefinition, type),
				CacheSharing.SHARED);
	}

	public final static String createQueryRegionName(
			JDBCTableDesc tableDefinition, QueryCacheType type) {
		if (type == null)
			throw new IllegalArgumentException(
					"Nie mozna nie zdefiniowac typu zapytan");
		return tableDefinition.getCacheRegionName() + "." + type.toString();

	}

	/**
	 * Usunęcie wszystkich elementów z kesza, którym zarządza instancja danego
	 * manager-a.
	 */
	public void clear() {
		if (this.cacheIsAllowed()) {
			try {
				getCache().clear();
				logger.trace("[{}] Kesz zostal wyczyszczony",
						getCacheRegionName());
			} catch (ObjectCacheErrorException e) {
				logger.error("[" + getCacheRegionName()
						+ "] Blad czyszczenia kesz-a", e);
			}
		}
	}

	/**
	 * @param cacheUsage
	 *            the {@link #cacheUsage} to set
	 */
	public void setCacheUsage(CacheUsage cacheUsage) {
		this.cacheUsage = cacheUsage;
	}

}
