package pl.slawas.scjdbc4jpa.jdbc.dao;

import java.lang.reflect.InvocationTargetException;
import java.util.Hashtable;

import pl.slawas.scjdbc4jpa.jdbc.JDBCEntityMetaData;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCColumnDesc;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCKeyDesc;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCTableDesc;
import pl.slawas.scjdbc4jpa.jdbc.exceptions.JDBCDaoTemplateException;
import pl.slawas.scjdbc4jpa.jdbc.utils.JDBCEntityParserLevel;
import pl.slawas.scjdbc4jpa.jdbc.utils.JDBCEntityValueParser;
import pl.slawas.scjdbc4jpa.jdbc.where.ConditionClauseEmbededIn;
import pl.slawas.scjdbc4jpa.jdbc.where.ConditionClauseIn;
import pl.slawas.scjdbc4jpa.jdbc.where.ConditionClauseSupport;
import pl.slawas.scjdbc4jpa.jdbc.where.ConditionOperator;

/**
 * 
 * JDBCDaoUtils
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.1 $
 * 
 */
public class JdbcDAOUtils {

	/**
	 * Tworzenie klauzuli IN dla listy identyfikatorów
	 * 
	 * @param metadata
	 *            metadane związane z daną encją
	 * @param idList
	 *            lista identyfikatorów encji
	 * @return klauzula warunku zapytania
	 */
	public static ConditionClauseSupport createINConditionForIdList(
			JDBCEntityMetaData metadata, Object[] idList) {
		JDBCTableDesc tableDesc = metadata
				.getTableDefinition(JDBCEntityParserLevel.THIRD);
		JDBCKeyDesc pk = tableDesc.getPrimaryKey();
		ConditionClauseSupport condition = null;
		if (pk.getColumns().size() > 1) {
			/*
			 * trzeba przetransformować macierz obiektów złozonych do prostej
			 * postaci
			 */
			Object[] values = new Object[pk.getColumns().size() * idList.length];
			JDBCColumnDesc[] pkColumns = pk.getColumnsArray();
			int valuesCounter = 0;
			for (int i = 0; i < idList.length; i++) {
				try {
					Hashtable<JDBCColumnDesc, Object> map = JDBCEntityValueParser
							.readKeyObject(pk, idList[i]);
					for (int jj = 0; jj < pkColumns.length; jj++) {
						values[valuesCounter] = map.get(pkColumns[jj]);
						valuesCounter++;
					}
				} catch (IllegalArgumentException e) {
					throw new JDBCDaoTemplateException(
							"Blad parsowania wartosci klucza z listy", e);
				} catch (IllegalAccessException e) {
					throw new JDBCDaoTemplateException(
							"Blad parsowania wartosci klucza z listy", e);
				} catch (InvocationTargetException e) {
					throw new JDBCDaoTemplateException(
							"Blad parsowania wartosci klucza z listy", e);
				}
			}
			condition = new ConditionClauseEmbededIn(ConditionOperator.AND,
					pkColumns, values, true);
		} else {
			condition = new ConditionClauseIn(ConditionOperator.AND, pk
					.getColumns().get(0), idList, true);
		}
		return condition;
	}

}
