package pl.slawas.scjdbc4jpa.jdbc.ddl;

import java.io.Serializable;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import pl.slawas.scjdbc4jpa.dialect.Dialect;

/**
 * 
 * SequenceCreator prosty kreator sekwencji. Jego implememntacja zakłada, że
 * sekwencja bedzie tworzona tylko dla engine Oracle.
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.4 $
 * 
 */
public class SequenceCreator implements Serializable {

	private static final long serialVersionUID = -2060989233114114637L;

	final static private Logger logger = LoggerFactory
			.getLogger(SequenceCreator.class);

	private final Dialect dialect;

	/**
	 * @param dialect
	 *            dialekt, na razie obsługuję tylko dilaekt
	 *            {@link Dialect#ORACLE}
	 */
	public SequenceCreator(Dialect dialect) {
		this.dialect = dialect;
	}

	public void createSequence(Connection conn, String sequenceName)
			throws SQLException {

		if (dialect.equals(Dialect.ORACLE)) {
			logger.debug("Tworzenie sekwencji {}", sequenceName);
			CallableStatement call = conn.prepareCall("DROP SEQUENCE "
					+ sequenceName);
			try {
				call.execute();
			} catch (SQLException e) {
				logger.error(
						"Operacja usuniecia sekwencji zakonczona niepowodzeniem.\n Przyczyna: ",
						e.getMessage());
			}
			StringBuffer createClause = new StringBuffer("CREATE SEQUENCE "
					+ sequenceName);
			call = conn.prepareCall(createClause.toString());
			call.execute();
			call.close();
			logger.debug("Sekwencja {} zostala utworzona", sequenceName);
		} else {
			throw new SQLException(
					"Tworzenie sekwencji supportowane jest tylko dla dialektu ORACLE");
		}
	}
}
