package pl.slawas.scjdbc4jpa.jdbc.dml;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import pl.slawas.scjdbc4jpa.jdbc.WhereClause;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCColumnDesc;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCTableDesc;
import pl.slawas.scjdbc4jpa.jdbc.types.SimpleSQLType;
import pl.slawas.scjdbc4jpa.jdbc.where.BindParam;
import pl.slawas.scjdbc4jpa.jdbc.where.ConditionClauseSupport;

public class Update extends DmlQuerySupport {

	final static private Logger logger = LoggerFactory.getLogger(Update.class);

	private static final long serialVersionUID = 1L;

	public Update(JDBCTableDesc table) {
		super(table, DataModificationType.UPDATE);
	}

	@Override
	protected StringBuffer init() {
		StringBuffer jdbcUpdateSQL = new StringBuffer("UPDATE ");
		jdbcUpdateSQL.append(table.getName() + " SET");
		this.bindParamsMapper = new Hashtable<String, Vector<BindParam>>();

		Enumeration<String> columns = this.table.getColumnKeys();
		int position = 0;
		while (columns.hasMoreElements()) {
			String entityFieldName = columns.nextElement();
			JDBCColumnDesc column = table.getColumn(entityFieldName);
			logger.trace("Kolumna: {}", column.toString());
			switch (column.getSource()) {
			case MAIN:
				jdbcUpdateSQL.append((position == 0 ? "\n  " : "\n ,")
						+ column.getName() + " = ?");
				BindParam bindParam = new BindParam(position + 1,
						SimpleSQLType.get(column.getColumnType()), null);
				if (this.bindParamsMapper.containsKey(entityFieldName)) {
					this.bindParamsMapper.get(entityFieldName).add(bindParam);
				} else {
					Vector<BindParam> b = new Vector<BindParam>();
					b.add(bindParam);
					this.bindParamsMapper.put(entityFieldName, b);
				}
				position++;
				break;
			default:
				continue;
			}
		}

		WhereClause where = new WhereClause(table.getPrimaryKey());
		jdbcUpdateSQL.append("\n " + where.getClause());
		Vector<BindParam> whereParams = where.getBindParams();
		ConditionClauseSupport[] conditionsArray = where.getConditionsArray();
		for (BindParam whereParam : whereParams) {
			int whereposition = whereParam.getParameterIndex();
			BindParam bindParam = new BindParam(position + whereposition,
					whereParam.getType(), null);
			String entityFieldName = table.getColumnByDbColumnName(
					conditionsArray[whereposition - 1].getParam().getField())
					.getEntityFieldName();
			if (this.bindParamsMapper.containsKey(entityFieldName)) {
				this.bindParamsMapper.get(entityFieldName).add(bindParam);
			} else {
				Vector<BindParam> b = new Vector<BindParam>();
				b.add(bindParam);
				this.bindParamsMapper.put(entityFieldName, b);
			}
		}

		return jdbcUpdateSQL;
	}

}
