package pl.slawas.scjdbc4jpa.jdbc.sort;

import java.io.Serializable;

import pl.slawas.scjdbc4jpa.jdbc.SelectQuery;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCColumnDesc;
import pl.slawas.scjdbc4jpa.jdbc.sql.functions._IFunction;

/**
 * 
 * SortCondition warunek sortowania
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.5 $
 * 
 */
public class SortCondition implements Serializable {

	private static final long serialVersionUID = 7964517652654345913L;

	protected SortParam sortParam;

	protected String fieldName;

	public SortCondition() {
		super();
	}

	/**
	 * 
	 * @param fieldName
	 * @param sortParam
	 */
	public SortCondition(String fieldName, SortParam sortParam) {
		super();
		this.fieldName = fieldName;
		this.sortParam = sortParam;
	}

	/**
	 * 
	 * @param column
	 * @param sortParam
	 */
	public SortCondition(JDBCColumnDesc column, SortParam sortParam) {
		super();
		this.fieldName = SelectQuery.getFieldName(column);
		this.sortParam = sortParam;
	}

	/**
	 * 
	 * @param column
	 */
	public SortCondition(JDBCColumnDesc column) {
		super();
		this.sortParam = SortParam.ASC;
		this.fieldName = SelectQuery.getFieldName(column);
	}

	/**
	 * 
	 * @param function
	 * @param sortParam
	 */
	public SortCondition(_IFunction function, SortParam sortParam) {
		super();
		this.fieldName = function.getClause();
		this.sortParam = sortParam;
	}

	/**
	 * 
	 * @param function
	 */
	public SortCondition(_IFunction function) {
		super();
		this.sortParam = SortParam.ASC;
		this.fieldName = function.getClause();
	}

	@Override
	public String toString() {
		return "SortCondition [fieldName=" + fieldName + ", sortParam="
				+ sortParam + "]";
	}

	/**
	 * Klauzula warunku sortowania
	 * 
	 * @return klauzula
	 */
	public String getClause() {
		return fieldName + " " + sortParam;
	}

}
