package pl.slawas.scjdbc4jpa.jdbc.sql.functions;

import java.io.Serializable;

/**
 * 
 * FunctionSupport klasa abstrakcyjna dla funklcji SQL
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.1 $
 * 
 */
@SuppressWarnings("serial")
public abstract class FunctionSupport implements _IFunction, Serializable {

	protected final String name;

	protected final String alias;

	protected final String[] arguments;

	private final Class<?> returnType;

	/**
	 * Konstruktor, za pomocą ktorego budowana jest funkcja, ze zdefiniowanym
	 * przez użytkownika typem wyniku funkcji.
	 * 
	 * @param returnType
	 *            klasa wyniku funkcji
	 * @param name
	 *            nazwa funkcji
	 * @param alias
	 *            nazwa kolumny w wyniku zapytania SQL pod jaką występować
	 *            bedzie wynik funkcji
	 * @param arguments
	 *            argumenty funkcji
	 */
	protected FunctionSupport(Class<?> returnType, String name, String alias,
			String[] arguments) {
		this.returnType = returnType;
		this.name = name;
		this.alias = alias;
		this.arguments = arguments;
	}

	public String getClause() {
		StringBuffer clause = new StringBuffer();
		clause.append(name + "(");
		int i = 0;
		for (String argument : arguments) {
			clause.append((i != 0 ? "," : "") + argument);
			i++;
		}
		clause.append(")");
		return clause.toString();
	}

	public Class<?> getReturnType() {
		return returnType;
	}

	public String getName() {
		return name;
	}

	public String getAlias() {
		return this.alias;
	}

	public String[] getArguments() {
		return this.arguments;
	}

}
