package pl.slawas.scjdbc4jpa.jdbc.sql.functions;

import pl.slawas.scjdbc4jpa.jdbc.sql.functions.FunctionSupport;

/**
 * 
 * NoneSupport - 'sztuczna' funkcja obsługująca kolumny nie zdefiniowane w
 * encji. Funkcja łączy podane argument w łańcuch wartości odseparowanych
 * znakiem spacji (' ').
 * <p>
 * Przykład:
 * </p>
 * 
 * <pre>
 * String[] arguments = new String[] { &quot;'test1'&quot; };
 * _IFunction function = new pl.slawas.jdbc.sql.functions.hsql.None(String.class,
 * 		&quot;test&quot;, arguments);
 * String clause = function.getClause();
 * // wartosc clause: 'test1'
 * // podczas uzycia w zapytaniu: 'test1' as test
 * arguments = new String[] { &quot;nazwa_kolumny&quot; };
 * function = new pl.slawas.jdbc.sql.functions.hsql.None(String.class, &quot;test&quot;,
 * 		arguments);
 * clause = function.getClause();
 * // wartosc clause: nazwa_kolumny
 * // podczas uzycia w zapytaniu: nazwa_kolumny as test
 * arguments = new String[] { &quot;count(*)&quot; };
 * function = new pl.slawas.jdbc.sql.functions.hsql.None(Integer.class, &quot;ile&quot;,
 * 		arguments);
 * clause = function.getClause();
 * // wartosc clause: count(*)
 * // podczas uzycia w zapytaniu: count(*) as ile
 * arguments = new String[] { &quot;nazwa_kolumny&quot;, &quot;=&quot;, &quot;'test'&quot; };
 * function = new pl.slawas.jdbc.sql.functions.hsql.None(Integer.class, &quot;ile&quot;,
 * 		arguments);
 * clause = function.getClause();
 * // wartosc clause: nazwa_kolumny = 'test'
 * </pre>
 * 
 * @see Function#None
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.1 $
 * 
 */
@SuppressWarnings({ "serial" })
public abstract class NoneSupport extends FunctionSupport {

	/**
	 * Konstruktor 'sztucznej' funkcji
	 * 
	 * @param returnType
	 *            typ wyniku funkcji
	 * @param name
	 *            nazwa funkcji
	 * @param alias
	 *            nazwa kolumny w wyniku zapytania SQL pod jaką występować
	 *            bedzie wynik funkcji
	 * @param arguments
	 *            argumenty funkcji
	 */
	public NoneSupport(Class<?> returnType, String name, String alias,
			String[] arguments) {
		super(returnType, name, alias, arguments);
	}

	@Override
	public String getClause() {
		StringBuffer clause = new StringBuffer();
		for (int i = 0; i < arguments.length; i++) {
			clause.append((i != 0 ? " " : "") + arguments[i]);
		}
		return clause.toString();
	}

}
