package pl.slawas.scjdbc4jpa.jdbc.where;

/**
 * 
 * ConditionClauseBool warunek zlozony, suma logiczna, budowany z listy różnych
 * innych warunków.
 * 
 * *
 * <p>
 * Przykład:
 * </p>
 * 
 * <pre>
 * expectedCondition = &quot;AND (&quot; + &quot;chi.nameDay = TO_DATE(?,'YYYY-MM-DD HH24:MI') &quot;
 * 		+ &quot;OR chi.nameDay != TO_DATE('2010-10-13 12:00','YYYY-MM-DD HH24:MI') &quot;
 * 		+ &quot;AND chi.age &lt;= TO_NUMBER('234567') &quot;
 * 		+ &quot;AND mother_1.PARENT_NAME &lt;= ('Barbara' || ' ' || 'Cichy') &quot;
 * 		+ &quot;AND chi.name LIKE ('%' || ? || '%') &quot;
 * 		+ &quot;AND chi.nameDay IS NOT NULL &quot; + &quot;AND chi.nameDay BETWEEN ? AND ?&quot;
 * 		+ &quot;)&quot;;
 * ConditionClauseBool tested8 = new ConditionClauseBool(ConditionOperator.AND,
 * 		new ConditionClause[] { tested1, tested2, tested3, tested4, tested5,
 * 				tested6, tested7 });
 * logger.debug(tested8.getClause());
 * assertEquals(&quot;Blad wygenerowania klauzuli warunku&quot;, expectedCondition,
 * 		tested8.getClause());
 * </pre>
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.2 $
 * 
 */
public class ConditionClauseBool extends ConditionClauseSupport {

	private static final long serialVersionUID = 6801361643123612029L;

	private ConditionClauseSupport[] clauses;

	/**
	 * @param conditionOperator
	 *            oprator warunku (jedna z wartości {@link ConditionOperator})
	 * @param clauses
	 *            lista warunków, które zostaną połączone i ich suma logiczna
	 *            musi być spełniona
	 */
	public ConditionClauseBool(ConditionOperator conditionOperator,
			ConditionClauseSupport[] clauses) {
		super(null, Boolean.class);
		init(conditionOperator, clauses);
	}

	private void init(ConditionOperator conditionOperator,
			ConditionClauseSupport[] clauses) {

		this.setConditionOperator(conditionOperator);
		this.setConditionType(ConditionType.BOOL);
		this.setParamOperator(ParamOperatorBool.BOOL);

		if (clauses == null || clauses.length == 0) {
			throw new IllegalArgumentException("Lista klauzul warunków, "
					+ "nie moze byc pusta");
		}
		this.clauses = clauses;
		ConditionParam[] params = new ConditionParam[clauses.length];
		for (int i = 0; i < this.clauses.length; i++) {
			params[i] = this.clauses[i].getParam();
		}
		this.getParam().setParams(params);

	}

	@Override
	public String getClause() {
		/* ustalenie operatorów */
		Operators operators = this.getOperators();
		String operator = operators.getOperator();
		ConditionOperator conditionOperator = operators.getConditionOperator();

		/* tworzenie klauzuli */
		StringBuffer out = new StringBuffer();
		// ConditionParam param = getParam();
		out.append(conditionOperator);
		out.append(" " + operator);
		out.append("(");
		for (int i = 0; i < this.clauses.length; i++) {
			String subClause = clauses[i].getClause();
			if (i == 0) {
				if (subClause.startsWith(ConditionOperator.AND.getOperator())) {
					subClause = subClause.substring(ConditionOperator.AND
							.getOperator().length() + 1);
				} else {
					subClause = subClause.substring(ConditionOperator.OR
							.getOperator().length() + 1);
				}
			} else {
				out.append(" ");
			}
			out.append(subClause);
		}
		out.append(")");

		return out.toString();
	}

	/**
	 * @return the clauses
	 */
	public ConditionClauseSupport[] getClauses() {
		return clauses;
	}

}
