package pl.slawas.scjdbc4jpa.jdbc.where;

import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCColumnDesc;

/**
 * 
 * ConditionClauseNull warunek zapytania sprawdzający czy kolumna ma wartość
 * pustą ({@code NULL}).
 * <p>
 * Przykład:
 * </p>
 * 
 * <pre>
 * expectedCondition = &quot;AND chi.nameDay IS NOT NULL&quot;;
 * tested = new ConditionClauseNull(ConditionOperator.ANDNOT,
 * 		table.getColumn(&quot;nameDay&quot;));
 * logger.debug(tested.getClause());
 * assertEquals(&quot;Blad wygenerowania klauzuli warunku&quot;, expectedCondition,
 * 		tested.getClause());
 * </pre>
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.1 $
 * 
 */
public class ConditionClauseNull extends ConditionClauseSupport {

	private static final long serialVersionUID = 6801361643123612029L;

	/**
	 * Konstruktor warunku zapytania.
	 * 
	 * @param conditionOperator
	 *            oprator warunku (jedna z wartości {@link ConditionOperator})
	 * @param column
	 *            opis kolumny, której dotyczy warunek
	 */
	public ConditionClauseNull(ConditionOperator conditionOperator,
			JDBCColumnDesc column) {
		super(column);
		init(conditionOperator);
	}

	private void init(ConditionOperator conditionOperator) {

		this.setConditionOperator(conditionOperator);
		this.setConditionType(ConditionType.NULL);
		this.setConditionValue("NULL");
		this.setValueAsLiteral();
		this.setParamOperator(ParamOperatorNull.IS);

	}

}
