package pl.slawas.scjdbc4jpa.jdbc.where;

import java.io.Serializable;

import pl.slawas.scjdbc4jpa.jdbc.SelectQuery;
import pl.slawas.scjdbc4jpa.jdbc.beans.JDBCColumnDesc;
import pl.slawas.scjdbc4jpa.jdbc.types.SimpleSQLType;

public abstract class ConditionClauseSupport implements Serializable {

	private static final long serialVersionUID = -4979213340825727768L;

	private ConditionParam param = new ConditionParam();

	protected ConditionClauseSupport() {
	}

	protected ConditionClauseSupport(String fieldName, Class<?> fieldType) {
		this.setField(fieldName);
		this.setFieldType(fieldType);
	}

	protected ConditionClauseSupport(JDBCColumnDesc column) {
		this.setField(SelectQuery.getFieldName(column));
		this.setFieldType(column.getColumnType());
	}

	protected ConditionOperator conditionOperator;

	protected void overrideParam(ConditionParam param) {
		this.param = param;
	}

	protected void overrideParams(ConditionParam[] params) {
		this.getParam().setParams(params);
	}

	/**
	 * @return the param
	 */
	public ConditionParam getParam() {
		return param;
	}

	/**
	 * @param type
	 *            the type to set
	 */
	protected void setConditionType(ConditionType type) {
		this.param.setType(type);
	}

	protected void setConditionOperator(ConditionOperator conditionOperator) {
		this.conditionOperator = conditionOperator;
	}

	/**
	 * @param field
	 *            the field to set
	 */
	protected void setField(String field) {
		this.param.setField(field);
	}

	/**
	 * @param fieldType
	 *            the fieldType to set
	 */
	protected void setFieldType(Class<?> fieldType) {
		SimpleSQLType type = SimpleSQLType.get(fieldType);
		if (type == null)
			throw new IllegalArgumentException("Niepoprawny typ pola "
					+ fieldType.getName());

		this.param.setFieldType(type);
	}

	protected void setConditionValue(Object value) {
		this.param.setValue(value);
	}

	protected void setValueAsBinded() {
		this.param.setValueAsBinded();
	}

	protected void setValueAsLiteral() {
		this.param.setValueAsLiteral();
	}

	protected void setParamOperator(_IParamOperator paramOperator) {
		this.param.setOperator(paramOperator);
	}

	public String getClause() {
		/* ustalenie operatorów */
		Operators operators = this.getOperators();
		String operator = operators.getOperator();
		ConditionOperator conditionOperator = operators.getConditionOperator();

		/* tworzenie klauzuli */
		StringBuffer out = new StringBuffer();
		ConditionParam param = getParam();
		out.append(conditionOperator);
		out.append(" " + param.getField());
		out.append(" " + operator);

		if (param.isBindedValue()) {
			out.append(" ?");
			return out.toString();
		}

		if (param.getValue() != null && param.getValue() instanceof String) {
			out.append(" " + (String) param.getValue());
		} else
			throw new IllegalArgumentException(
					"Jezeli uzywamy literalow to wartosc parametru musi byc nie pustym obiektem klasy java.lang.String ");
		return out.toString();
	}

	public Operators getOperators() {

		String operator = param.getOperator().getOperator();
		ConditionOperator conditionOperator = this.conditionOperator;
		switch (conditionOperator) {
		case ANDNOT:
			operator = param.getOperator().getNotOperator();
			conditionOperator = ConditionOperator.AND;
			break;
		case ORNOT:
			operator = param.getOperator().getNotOperator();
			conditionOperator = ConditionOperator.OR;
			break;
		default:
			break;
		}

		return new Operators(operator, conditionOperator);
	}

	public static class Operators {

		private final String operator;

		private final ConditionOperator conditionOperator;

		/**
		 * @param operator
		 * @param conditionOperator
		 */
		private Operators(String operator, ConditionOperator conditionOperator) {
			super();
			this.operator = operator;
			this.conditionOperator = conditionOperator;
		}

		/**
		 * @return the operator
		 */
		public String getOperator() {
			return operator;
		}

		/**
		 * @return the conditionOperator
		 */
		public ConditionOperator getConditionOperator() {
			return conditionOperator;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime
					* result
					+ ((conditionOperator == null) ? 0 : conditionOperator
							.hashCode());
			result = prime * result
					+ ((operator == null) ? 0 : operator.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			Operators other = (Operators) obj;
			if (conditionOperator == null) {
				if (other.conditionOperator != null)
					return false;
			} else if (!conditionOperator.equals(other.conditionOperator))
				return false;
			if (operator == null) {
				if (other.operator != null)
					return false;
			} else if (!operator.equals(other.operator))
				return false;
			return true;
		}

	}

}
