package pl.slawas.scjdbc4jpa.jdbc.where;

import java.io.Serializable;
import java.util.Arrays;

import pl.slawas.scjdbc4jpa.jdbc.sql.functions._IFunction;
import pl.slawas.scjdbc4jpa.jdbc.types.SimpleSQLType;

/**
 * 
 * ConditionParam obiekt parametrów warunku zapytania
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.3 $
 * 
 */
public class ConditionParam implements Serializable {

	private static final long serialVersionUID = -7492912988120274823L;

	private ConditionType type;

	private _IParamOperator operator;

	private String field;

	private SimpleSQLType fieldType;

	private Object value;

	private boolean isCaseSensitive = true;

	/** Obiekt funkcji przetwarzająca wartość parametru */
	private _IFunction function;

	/**
	 * preferujemy zapytania ze zmiennymi bindowanymi.
	 */
	private boolean isBindValue = true;

	/**
	 * Tylko dla zapytań typu range. Dolna granica zakresu.
	 */
	private Object lowerBound;

	/** Obiekt funkcji przetwarzająca wartość dolnej granicy parametru */
	private _IFunction lowerFunction;

	/**
	 * Tylko dla zapytań typu range. Górna granica zakresu.
	 */
	private Object upperBound;

	/** Obiekt funkcji przetwarzająca wartość gornej granicy parametru */
	private _IFunction upperFunction;

	/**
	 * Tylko dla zapytań typu range. Czy przedział ma być lewostronnie
	 * domknięty.
	 */
	private boolean inclusiveLower = false;

	/**
	 * Tylko dla zapytań typu range. Czy przedział ma być prawostronnie
	 * domknięty.
	 */
	private boolean inclusiveUpper = false;

	private ConditionParam[] params;

	/**
	 * @return the type
	 */
	public ConditionType getType() {
		return type;
	}

	/**
	 * @param type
	 *            the type to set
	 */
	void setType(ConditionType type) {
		this.type = type;
	}

	public _IParamOperator getOperator() {
		return operator;
	}

	void setOperator(_IParamOperator operator) {
		this.operator = operator;
	}

	/**
	 * @return the field
	 */
	public String getField() {
		return field;
	}

	/**
	 * @param field
	 *            the field to set
	 */
	void setField(String field) {
		this.field = field;
	}

	/**
	 * @return the fieldType
	 */
	public SimpleSQLType getFieldType() {
		return fieldType;
	}

	/**
	 * @param fieldType
	 *            the fieldType to set
	 */
	void setFieldType(SimpleSQLType fieldType) {
		this.fieldType = fieldType;
	}

	/**
	 * @return the params
	 */
	public ConditionParam[] getParams() {
		return params;
	}

	/**
	 * @param params
	 *            the params to set
	 */
	void setParams(ConditionParam[] params) {
		this.params = params;
	}

	/**
	 * Metoda mówiąca o tym czy warunek jest wartością bindowaną
	 * 
	 * @return {@code true} jeżeli {@link #literalValue} jest puste
	 */
	public boolean isBindedValue() {
		return this.isBindValue;
	}

	/**
	 * Metoda mówiąca o tym czy warunek jest wartością literalną (literałem)
	 * 
	 * @return {@code true} jeżeli {@link #literalValue} nie jest puste
	 */
	public boolean isLiteralValue() {
		return !this.isBindValue;
	}

	/**
	 * @param isBindValue
	 *            the isBindValue to set
	 */
	void setValueAsBinded() {
		this.isBindValue = true;
	}

	/**
	 * @param isBindValue
	 *            the isBindValue to set
	 */
	void setValueAsLiteral() {
		this.isBindValue = false;
	}

	/**
	 * @return the value
	 */
	public Object getValue() {
		return value;
	}

	/**
	 * @param value
	 *            the value to set
	 */
	void setValue(Object value) {
		this.value = value;
	}

	/**
	 * @return the lowerBound
	 */
	public Object getLowerBound() {
		return lowerBound;
	}

	/**
	 * @param lowerBound
	 *            the lowerBound to set
	 */
	void setLowerBound(Object lowerBound) {
		this.lowerBound = lowerBound;
	}

	/**
	 * @return the upperBound
	 */
	public Object getUpperBound() {
		return upperBound;
	}

	/**
	 * @param upperBound
	 *            the upperBound to set
	 */
	void setUpperBound(Object upperBound) {
		this.upperBound = upperBound;
	}

	/**
	 * @return the inclusiveLower
	 */
	public boolean isInclusiveLower() {
		return inclusiveLower;
	}

	/**
	 * @param inclusiveLower
	 *            the inclusiveLower to set
	 */
	void setInclusiveLower(boolean inclusiveLower) {
		this.inclusiveLower = inclusiveLower;
	}

	/**
	 * @return the inclusiveUpper
	 */
	public boolean isInclusiveUpper() {
		return inclusiveUpper;
	}

	/**
	 * @param inclusiveUpper
	 *            the inclusiveUpper to set
	 */
	void setInclusiveUpper(boolean inclusiveUpper) {
		this.inclusiveUpper = inclusiveUpper;
	}

	/**
	 * @return the function
	 */
	public _IFunction getFunction() {
		return function;
	}

	/**
	 * @param function
	 *            the function to set
	 */
	void setFunction(_IFunction function) {
		this.function = function;
	}

	/**
	 * @return the isCaseSensitive
	 */
	public boolean isCaseSensitive() {
		return isCaseSensitive;
	}

	/**
	 * @param isCaseSensitive
	 *            the isCaseSensitive to set
	 */
	void setCaseSensitive(boolean isCaseSensitive) {
		this.isCaseSensitive = isCaseSensitive;
	}

	/**
	 * @return the lowerFunction
	 */
	public _IFunction getLowerFunction() {
		return lowerFunction;
	}

	/**
	 * @param lowerFunction
	 *            the lowerFunction to set
	 */
	void setLowerFunction(_IFunction lowerFunction) {
		this.lowerFunction = lowerFunction;
	}

	/**
	 * @return the upperFunction
	 */
	public _IFunction getUpperFunction() {
		return upperFunction;
	}

	/**
	 * @param upperFunction
	 *            the upperFunction to set
	 */
	void setUpperFunction(_IFunction upperFunction) {
		this.upperFunction = upperFunction;
	}

	@Override
	public String toString() {
		return "ConditionParam [field=" + field + ", fieldType=" + fieldType
				+ ", function=" + function + ", inclusiveLower="
				+ inclusiveLower + ", inclusiveUpper=" + inclusiveUpper
				+ ", isBindValue=" + isBindValue + ", isCaseSensitive="
				+ isCaseSensitive + ", lowerBound=" + lowerBound
				+ ", lowerFunction=" + lowerFunction + ", operator=" + operator
				+ ", params=" + Arrays.toString(params) + ", type=" + type
				+ ", upperBound=" + upperBound + ", upperFunction="
				+ upperFunction + ", value=" + value + "]";
	}

}
