package pl.slawas.scjdbc4jpa.jndi;

import java.util.Enumeration;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.jndi.JndiTemplate;

import pl.slawas.scjdbc4jpa.core.ScDefaults;
import pl.slawas.scjdbc4jpa.jdbc.types.SimpleSQLTypeHelper;

/**
 * JNDI Factory niezbedny do przeprowadznia testów z TransactionManager-em
 * zdefiniowanym w springu.
 * 
 * @author Slawomir Cichy slawas@slawas.pl
 * 
 */
public class JndiFactory extends JndiTemplate implements InitializingBean,
		BeanPostProcessor, ApplicationContextAware, ScDefaults {

	final protected Logger log = Logger.getLogger(getClass());

	private ApplicationContext appCtx;

	private String testJndiNameObject = "ScTestJndiNameObject";

	private boolean testingFactory = false;

	private String doTest = FALSE;

	public JndiFactory(Properties environment) throws NamingException {
		super(environment);
		String jndiInfo = "\n JNDI properties [";
		Enumeration<Object> keys = environment.keys();
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			String value = environment.getProperty(key);
			jndiInfo = jndiInfo + "\n	" + key + " = " + value;
			if (key.equals("java.naming.factory.initial"))
				System.setProperty(Context.INITIAL_CONTEXT_FACTORY, value);
			if (key.equals("java.naming.provider.url"))
				System.setProperty(Context.PROVIDER_URL, value);

		}
		log.info(jndiInfo + "\n	...]");
	}

	public Object postProcessAfterInitialization(Object bean, String arg1)
			throws BeansException {
		return bean;
	}

	public Object postProcessBeforeInitialization(Object bean, String arg1)
			throws BeansException {
		return bean;
	}

	public void setApplicationContext(ApplicationContext arg0)
			throws BeansException {
		this.appCtx = arg0;
	}

	public void afterPropertiesSet() throws Exception {
		ContextBean cb = ContextBean.getInstance();
		cb.setApplicationContext(appCtx);
		if (this.testingFactory) {
			Context ictx = new InitialContext();
			ictx.rebind(this.testJndiNameObject, new JndiTestObject(
					this.testJndiNameObject));
		}
	}

	/**
	 * @param testJndiNameObject
	 *            the testJndiNameObject to set
	 */
	public void setTestJndiNameObject(String testJndiNameObject) {
		this.testJndiNameObject = testJndiNameObject;
	}

	/**
	 * @param doTest
	 *            the doTest to set
	 */
	public void setDoTest(String doTest) {
		this.doTest = doTest;
		this.testingFactory = SimpleSQLTypeHelper.validateBooleanValue(
				"doTest", this.doTest, log);
	}

}
