package pl.slawas.scjdbc4jpa.jpa;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.net.URL;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceException;
import javax.persistence.spi.PersistenceProvider;
import javax.persistence.spi.PersistenceUnitInfo;
import javax.sql.DataSource;

import org.springframework.beans.factory.BeanClassLoaderAware;
import org.springframework.context.ResourceLoaderAware;
import org.springframework.context.weaving.LoadTimeWeaverAware;
import org.springframework.core.io.ResourceLoader;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.support.PersistenceExceptionTranslator;
import org.springframework.instrument.classloading.LoadTimeWeaver;
import org.springframework.jdbc.datasource.lookup.SingleDataSourceLookup;
import org.springframework.orm.jpa.DefaultJpaDialect;
import org.springframework.orm.jpa.EntityManagerFactoryInfo;
import org.springframework.orm.jpa.EntityManagerFactoryUtils;
import org.springframework.orm.jpa.JpaDialect;
import org.springframework.orm.jpa.persistenceunit.DefaultPersistenceUnitManager;
import org.springframework.orm.jpa.persistenceunit.PersistenceUnitManager;
import org.springframework.orm.jpa.persistenceunit.PersistenceUnitPostProcessor;
import org.springframework.util.ClassUtils;

import pl.slawas.scjdbc4jpa.core.ScEntityManager;
import pl.slawas.scjdbc4jpa.core.ScEntityManagerFactorySupport;

public class JpaContainerEntityManagerFactoryBean extends
		ScEntityManagerFactorySupport implements ResourceLoaderAware,
		LoadTimeWeaverAware, PersistenceExceptionTranslator,
		EntityManagerFactoryInfo, BeanClassLoaderAware {

	private static final long serialVersionUID = 6133708511284373600L;

	/** context classloader */
	private ClassLoader classLoader;

	/** Raw EntityManagerFactory as returned by the PersistenceProvider */
	public _JpaEntityManagerFactory nativeEntityManagerFactory;

	private _JpaEntityManagerFactory entityManagerFactory;

	/** **************************************************************** */

	protected final DefaultPersistenceUnitManager dpum = new DefaultPersistenceUnitManager();

	protected PersistenceUnitManager persistenceUnitManager;

	protected String persistenceUnitName;

	protected String persistenceFileName;

	protected PersistenceUnitInfo persistenceUnitInfo;

	private PersistenceProvider persistenceProvider;

	/** **************************************************************** */

	public void setPersistenceUnitManager(
			PersistenceUnitManager persistenceUnitManager) {
		this.persistenceUnitManager = persistenceUnitManager;
	}

	/**
	 * @return the persistenceUnitName
	 */
	public String getPersistenceUnitName() {
		return persistenceUnitName;
	}

	/**
	 * @param persistenceUnitName
	 *            the persistenceUnitName to set
	 */
	public void setPersistenceUnitName(String persistenceUnitName) {
		this.persistenceUnitName = persistenceUnitName;
	}

	/**
	 * @return the persistenceFileName
	 */
	public String getPersistenceFileName() {
		return persistenceFileName;
	}

	/**
	 * @param persistenceFileName
	 *            the persistenceFileName to set
	 */
	public void setPersistenceFileName(String persistenceFileName) {
		this.persistenceFileName = persistenceFileName;
	}

	public void setPersistenceXmlLocation(String persistenceXmlLocation) {
		this.dpum
				.setPersistenceXmlLocations(new String[] { persistenceXmlLocation });
	}

	/**
	 * @return the persistenceUnitInfo
	 */
	public PersistenceUnitInfo getPersistenceUnitInfo() {
		return persistenceUnitInfo;
	}

	public DataSource getDataSource() {
		if (this.persistenceUnitInfo != null) {
			return this.persistenceUnitInfo.getNonJtaDataSource();
		}
		return this.dpum.getDefaultDataSource();
	}

	/**
	 * @param dataSource
	 *            the dataSource to set
	 */
	protected void setterOfDataSourceMethodImplementation(DataSource dataSource) {
		this.dpum.setDataSourceLookup(new SingleDataSourceLookup(dataSource));
		this.dpum.setDefaultDataSource(dataSource);
	}

	/** **************************************************************** */

	public _JpaEntityManagerFactory getObject() throws Exception {
		return this.entityManagerFactory;
	}

	@SuppressWarnings("rawtypes")
	public Class getObjectType() {
		return (this.entityManagerFactory != null ? this.entityManagerFactory
				.getClass() : _JpaEntityManagerFactory.class);
	}

	public void destroy() throws Exception {
		logger.info("Closing JPA EntityManagerFactory for persistence unit '"
				+ getPersistenceUnitName() + "'");
		this.entityManagerFactory.close();
	}

	public DataAccessException translateExceptionIfPossible(RuntimeException ex) {
		return EntityManagerFactoryUtils
				.convertJpaAccessExceptionIfPossible(ex);
	}

	public Class<? extends EntityManager> getEntityManagerInterface() {
		return ScEntityManager.class;
	}

	public JpaDialect getJpaDialect() {
		//TODO Zrobić obsługę dialektów
		return new DefaultJpaDialect();
	}

	public EntityManagerFactory getNativeEntityManagerFactory() {
		return this.nativeEntityManagerFactory;
	}

	public PersistenceProvider getPersistenceProvider() {
		if (this.persistenceProvider == null)
			this.persistenceProvider = new JpaPersistence(this.getDataSource());
		return this.persistenceProvider;
	}

	public ClassLoader getBeanClassLoader() {
		return this.classLoader;
	}

	public void setBeanClassLoader(ClassLoader classLoader) {
		this.classLoader = classLoader;
	}

	public void setResourceLoader(ResourceLoader arg0) {
		this.dpum.setResourceLoader(arg0);
	}

	public void setLoadTimeWeaver(LoadTimeWeaver arg0) {
		this.dpum.setLoadTimeWeaver(arg0);
	}

	public void setPersistenceUnitPostProcessors(
			PersistenceUnitPostProcessor[] postProcessors) {
		this.dpum.setPersistenceUnitPostProcessors(postProcessors);
	}

	public void afterPropertiesSet() throws Exception {
		super.afterPropertiesSet();
		this.persistenceProvider = getPersistenceProvider();
		this.nativeEntityManagerFactory = createNativeEntityManagerFactory();
		this.entityManagerFactory = createEntityManagerFactoryProxy(this.nativeEntityManagerFactory);
	}

	/**
	 * Create a proxy of the given ScEntityManagerFactory. We do this to be able
	 * to return transaction-aware proxies for application-managed
	 * ScEntityManagers, and to introduce the NamedEntityManagerFactory
	 * interface
	 * 
	 * @param emf
	 *            ScEntityManagerFactory as returned by the persistence provider
	 * @return proxy entity manager
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	protected _JpaEntityManagerFactory createEntityManagerFactoryProxy(
			_JpaEntityManagerFactory emf) {
		ClassLoader cl = getClass().getClassLoader();
		Set ifcs = new LinkedHashSet();
		ifcs.addAll(ClassUtils.getAllInterfacesForClassAsSet(emf.getClass(), cl));
		ifcs.add(_JpaEntityManagerFactory.class);
		return (_JpaEntityManagerFactory) Proxy.newProxyInstance(cl,
				(Class[]) ifcs.toArray(new Class[ifcs.size()]),
				new ManagedEntityManagerFactoryInvocationHandler(emf, this));
	}

	/**
	 * Dynamic proxy invocation handler proxying an EntityManagerFactory to
	 * return a proxy EntityManager if necessary from createEntityManager()
	 * methods.
	 */
	private static class ManagedEntityManagerFactoryInvocationHandler implements
			InvocationHandler {

		private final _JpaEntityManagerFactory targetEntityManagerFactory;

		private final EntityManagerFactoryInfo entityManagerFactoryInfo;

		public ManagedEntityManagerFactoryInvocationHandler(
				_JpaEntityManagerFactory targetEmf,
				EntityManagerFactoryInfo emfInfo) {

			this.targetEntityManagerFactory = targetEmf;
			this.entityManagerFactoryInfo = emfInfo;

		}

		public Object invoke(Object proxy, Method method, Object[] args)
				throws Throwable {
			try {
				Object retVal = method.invoke(this.targetEntityManagerFactory,
						args);
				if (retVal instanceof ScEntityManager) {
					ScEntityManager rawEntityManager = (ScEntityManager) retVal;
					retVal = JpaExtendedEntityManagerUtils
							.createApplicationManagedEntityManager(
									rawEntityManager,
									this.entityManagerFactoryInfo);
				}
				return retVal;
			} catch (InvocationTargetException ex) {
				throw ex.getTargetException();
			}
		}
	}

	protected _JpaEntityManagerFactory createNativeEntityManagerFactory()
			throws PersistenceException {

		if ((persistenceUnitName == null) && (persistenceFileName == null))
			throw new IllegalArgumentException(
					"You must define 'persistenceUnitName' or 'persistenceFileName' parameter");

		if (persistenceFileName != null) {
			URL myLocation = JpaContainerEntityManagerFactoryBean.class
					.getResource(DEFAULT_PERSISTENCE_XML_DIR_LOCATION
							+ persistenceFileName);
			if (myLocation == null)
				throw new IllegalArgumentException(
						"Can not find persistence file '"
								+ DEFAULT_PERSISTENCE_XML_DIR_LOCATION
								+ persistenceFileName + "'");
			dpum.setPersistenceXmlLocations(new String[] { DEFAULT_PERSISTENCE_XML_URL_LOCATION
					+ persistenceFileName });
		}

		PersistenceUnitManager managerToUse = this.persistenceUnitManager;
		if (this.persistenceUnitManager == null) {
			this.dpum.afterPropertiesSet();
			managerToUse = this.dpum;
		}
		this.persistenceUnitInfo = (persistenceUnitName != null ? managerToUse
				.obtainPersistenceUnitInfo(persistenceUnitName) : managerToUse
				.obtainDefaultPersistenceUnitInfo());
		;
		Map<String, String> mapProperty = new HashMap<String, String>();
		mapProperty.put("batchSize", Integer.toString(getBatchSize()));
		mapProperty.put("batchCommitSize",
				Integer.toString(getBatchCommitSize()));
		return (_JpaEntityManagerFactory) this.getPersistenceProvider()
				.createContainerEntityManagerFactory(this.persistenceUnitInfo,
						mapProperty);
	}

}
