package pl.slawas.scjdbc4jpa.jpa;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.spi.PersistenceUnitInfo;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.ejb.Ejb3Configuration;
import org.hibernate.engine.Mapping;

import pl.slawas.scjdbc4jpa.core.ScEntityManager;
import pl.slawas.scjdbc4jpa.core.ScEntityManagerFactorySupport;
import pl.slawas.scjdbc4jpa.dialect.DefaultDialect;

public class JpaEntityManagerFactory extends ScEntityManagerFactorySupport
		implements _JpaEntityManagerFactory {

	private static final long serialVersionUID = 8519535422761322091L;

	final protected Log log = LogFactory.getLog(getClass());

	private final Ejb3Configuration ejb3Conf = new Ejb3Configuration();

	private final PersistenceUnitInfo persistenceUnitInfo;

	private EntityManagerFactory hibernateEntityManagerFactory;

	private Configuration configuration;

	private int batchSize = -1;

	private int batchCommitSize = DefaultBatchCommitSize;

	private Connection connection;

	private DataSource dataSource;

	private Mapping mapping;

	private EntityManagerFactory buildHibernateEntityManagerFactory() {
		return getHibernateEntityManagerFactory();
	}

	private EntityManagerFactory getHibernateEntityManagerFactory() {
		if (this.hibernateEntityManagerFactory == null
				|| !this.hibernateEntityManagerFactory.isOpen()) {
			this.configuration = ejb3Conf.configure(this.persistenceUnitInfo,
					null).getHibernateConfiguration();
			this.hibernateEntityManagerFactory = this.ejb3Conf
					.buildEntityManagerFactory();
		}
		return this.hibernateEntityManagerFactory;
	}

	public JpaEntityManagerFactory(PersistenceUnitInfo persistenceUnitInfo,
			int batchSize, int batchCommitSize, DataSource dataSource) {
		this.persistenceUnitInfo = persistenceUnitInfo;
		this.dataSource = dataSource;
		this.batchSize = batchSize;
		this.batchCommitSize = batchCommitSize;
	}

	public void close() {
		this.hibernateEntityManagerFactory.close();
		if (this.connection != null) {
			try {
				if (!this.connection.isClosed())
					this.connection.close();
			} catch (SQLException e) {
				log.warn("Closing connection error." + e.getMessage() + ". "
						+ e.getSQLState());
			}
			this.connection = null;
		}
	}

	public ScEntityManager createEntityManager() {
		return new DefaultDialect(getHibernateEntityManagerFactory()
				.createEntityManager(), batchSize, batchCommitSize, dataSource,
				getConfiguration(), getMapping());
	}

	@SuppressWarnings("rawtypes")
	public ScEntityManager createEntityManager(Map arg0) {
		return new DefaultDialect(getHibernateEntityManagerFactory()
				.createEntityManager(arg0), batchSize, batchCommitSize,
				dataSource, getConfiguration(), getMapping());
	}

	public boolean isOpen() {
		return getHibernateEntityManagerFactory().isOpen();
	}

	public int getBatchCommitSize() {
		return this.batchCommitSize;
	}

	public int getBatchSize() {
		return this.batchSize;
	}

	public Configuration getConfiguration() {
		if (this.configuration == null) {
			buildHibernateEntityManagerFactory();
		}
		return this.configuration;
	}

	public Mapping getMapping() {
		if (this.mapping == null && getConfiguration() != null) {
			this.mapping = getConfiguration().buildMapping();
		}
		return this.mapping;
	}

	public DataSource getDataSource() {
		return dataSource;
	}

	public Object getObject() throws Exception {
		log.info("JpaEntityManagerFactoryBean: getObject()");
		return this;
	}

	public Class<?> getObjectType() {
		return _JpaEntityManagerFactory.class;
	}

	public boolean isSingleton() {
		return true;
	}

	@Override
	public void destroy() throws Exception {
		logger.info("Zamykam JpaEntityManagerFactory...");
		this.close();

	}

	@Override
	protected void setterOfDataSourceMethodImplementation(DataSource dataSource) {
		this.dataSource = dataSource;
	}

}
