package pl.slawas.scjdbc4jpa.jpa;

import java.util.Enumeration;
import java.util.Map;
import java.util.Properties;

import javax.persistence.spi.PersistenceProvider;
import javax.persistence.spi.PersistenceUnitInfo;
import javax.sql.DataSource;

import org.apache.log4j.Logger;

import pl.slawas.scjdbc4jpa.core.ScDefaults;

public class JpaPersistence implements PersistenceProvider, ScDefaults {

	final protected Logger log = Logger.getLogger(getClass());

	private final DataSource standardDataSource;

	public JpaPersistence(DataSource standardDataSource) {
		this.standardDataSource = standardDataSource;
	}

	@SuppressWarnings("rawtypes")
	private void printPersistenceUnitInfo(
			PersistenceUnitInfo persistenceUnitInfo) {
		Properties pui;
		pui = persistenceUnitInfo.getProperties();
		String hibernateInfo = "\n Persistence unit info [\n";
		Enumeration keys = pui.keys();
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			String value = pui.getProperty(key);
			hibernateInfo = hibernateInfo + "	" + key + " = " + value + "\n";
		}

		hibernateInfo = hibernateInfo + "	...]";
		this.log.info(hibernateInfo);
	}

	@SuppressWarnings("rawtypes")
	public _JpaEntityManagerFactory createContainerEntityManagerFactory(
			PersistenceUnitInfo persistenceUnitInfo, Map propertyMap) {

		String batchSize = (propertyMap.get("batchSize") != null ? (String) propertyMap
				.get("batchSize") : Integer.toString(DefaultBatchSize));
		String batchCommitSize = (propertyMap.get("batchCommitSize") != null ? (String) propertyMap
				.get("batchCommitSize") : Integer
				.toString(DefaultBatchCommitSize));

		Properties pui = persistenceUnitInfo.getProperties();
		pui.setProperty("hibernate.jdbc.batch_size", batchSize);

		/* for presents info about persistence unit */
		printPersistenceUnitInfo(persistenceUnitInfo);

		return new JpaEntityManagerFactory(persistenceUnitInfo,
				Integer.parseInt(batchSize), Integer.parseInt(batchCommitSize),
				this.standardDataSource);

	}

	@SuppressWarnings("rawtypes")
	public _JpaEntityManagerFactory createEntityManagerFactory(String arg0,
			Map arg1) {
		throw new RuntimeException("NOT IMPLEMENTED");
	}

}
