package pl.slawas.scjdbc4jpa.jpa;

import java.sql.Connection;
import java.sql.SQLException;

import javax.persistence.EntityTransaction;

public class JpaTransaction implements EntityTransaction {

	private Connection conn;

	private String sqlErrorMessage;

	private int sqlErrorCode;

	private boolean active = false;

	private boolean rollbackOnly = false;

	private void cleanErrorStatus() {
		sqlErrorCode = 0;
		sqlErrorMessage = "OK";
	}

	public JpaTransaction(Connection conn) {
		this.conn = conn;
	}

	public void begin() {
		cleanErrorStatus();
		try {
			this.conn.setAutoCommit(false);
			this.active = true;
		} catch (SQLException e) {
			sqlErrorCode = 0 - e.getErrorCode();
			sqlErrorMessage = "[ORA" + sqlErrorCode + "] " + e.getMessage();
		}
	}

	public void commit() {
		cleanErrorStatus();
		try {
			conn.commit();
			this.active = false;
		} catch (SQLException e) {
			sqlErrorCode = 0 - e.getErrorCode();
			sqlErrorMessage = "[ORA" + sqlErrorCode + "] " + e.getMessage();
		}

	}

	public boolean getRollbackOnly() {
		return rollbackOnly;
	}

	public boolean isActive() {
		return this.active;
	}

	public void rollback() {
		cleanErrorStatus();
		try {
			conn.rollback();
			this.active = false;
		} catch (SQLException e) {
			sqlErrorCode = 0 - e.getErrorCode();
			sqlErrorMessage = "[ORA" + sqlErrorCode + "] " + e.getMessage();
		}
	}

	public void setRollbackOnly() {
		rollbackOnly = true;
	}

	/**
	 * @return the conn
	 */
	public Connection getConnection() {
		return conn;
	}

	/**
	 * @return the sqlErrorMessage
	 */
	public String getSqlErrorMessage() {
		return sqlErrorMessage;
	}

	/**
	 * @return the sqlErrorCode
	 */
	public int getSqlErrorCode() {
		return sqlErrorCode;
	}

}
