package pl.slawas.scjdbc4jpa.jpa.dao;

import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import pl.slawas.scjdbc4jpa.core.ScEntityManager;
import pl.slawas.scjdbc4jpa.core.ScObjectFactory;
import pl.slawas.scjdbc4jpa.jpa.mapping.RowValues;
import pl.slawas.scjdbc4jpa.jpa.mapping.Table;

public interface _IJpaDAO<Obj> {

	public void joinTransaction() throws SQLException;

	public Class<Obj> getPersistentClass();

	/**
	 * Pobieranie encji z bazy danych na podstawie wartości id (PK)
	 * 
	 * @param id
	 * @return
	 */
	public Obj find(Object id) throws SQLException;

	/**
	 * Wyszukiwanie wszystkich encji w bazie danych
	 * 
	 * @return
	 */
	public List<Obj> findAll() throws SQLException;

	/**
	 * Wyszukiwanie encji na podstawie zdefiniowanego w jej klasie zapytania i
	 * dowolna lista argumentow
	 * 
	 * @param namedQuery
	 * @param sqlParams
	 * @return
	 */
	@SuppressWarnings({ "rawtypes" })
	public List<Obj> find(String namedQuery, Map sqlParams) throws SQLException;

	public List<Obj> findByNativeQuery(String nativeQuery) throws SQLException;

	public boolean createFullTextIndex() throws SQLException;

	public List<Obj> search(String serachText, String[] indexedFieldList)
			throws SQLException;

	/**
	 * Wstawianie (tworzenie wiersza) encji do bazy danych
	 * 
	 * @param entityObject
	 */
	public void insert(Obj entityObject) throws SQLException;

	/**
	 * Wstawianie listy nowych encji ("bulk'owe" przetwarzanie wg. JPA) do bazy
	 * danych
	 * 
	 * @param entityObjects
	 * @param autoCommit
	 */
	public int insertList(List<Obj> entityObjects, boolean autoCommit)
			throws SQLException;

	/**
	 * Aktualizacja encji w bazie danych
	 * 
	 * @param entityObject
	 */
	public void update(Obj entityObject) throws SQLException;

	/**
	 * Aktualizacja listy zmienionych encji ("bulk'owe" przetwarzanie wg. JPA) w
	 * bazie danych
	 * 
	 * @note można używać dla wszystkich silników bazodanowych
	 * 
	 * @param entityObjects
	 * @param autoCommit
	 */
	public int updateList(List<Obj> entityObjects, boolean autoCommit)
			throws SQLException;

	/**
	 * Usuwanie encji z bazy danych
	 * 
	 * @param entityObject
	 */
	public void remove(Obj entityObject) throws SQLException;

	/**
	 * Usuwanie listy encji ("bulk'owe" przetwarzanie wg. JPA) w bazie danych
	 * 
	 * @param entityObjects
	 * @param autoCommit
	 */
	public int removeList(List<Obj> entityObjects, boolean autoCommit)
			throws SQLException;

	public Table getTableInfo() throws SQLException;

	public RowValues fetchValues(Obj entityObject) throws SQLException;

	/**
	 * @return the vendor
	 */
	public ScEntityManager getEntityManager();

	public void setEntityManager(ScEntityManager aem);

	public void setScObjectFactory(ScObjectFactory aof);

}