package pl.slawas.scjdbc4jpa.jpa.mapping;

import java.io.Serializable;
import java.lang.annotation.Annotation;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.hibernate.SessionFactory;
import org.hibernate.dialect.Dialect;
import org.hibernate.engine.Mapping;
import org.hibernate.impl.SessionImpl;
import org.hibernate.mapping.ForeignKey;
import org.hibernate.mapping.PersistentClass;
import org.hibernate.mapping.Property;
import org.hibernate.mapping.Value;
import org.hibernate.metadata.ClassMetadata;
import org.hibernate.type.AssociationType;
import org.hibernate.type.ComponentType;
import org.hibernate.type.Type;
import org.springframework.context.ApplicationContext;

import pl.slawas.helpers.Strings;
import pl.slawas.scjdbc4jpa.core.ScEntityManager;
import pl.slawas.scjdbc4jpa.jpa.annotations.DeleteHint;
import pl.slawas.scjdbc4jpa.jpa.annotations.InsertHint;
import pl.slawas.scjdbc4jpa.jpa.annotations.UpdateHint;
import pl.slawas.scjdbc4jpa.jpa.dao._IJpaDAO;

/**
 * 
 * Table definicja tabeli
 * 
 * @author Sławomir Cichy &lt;slawas@slawas.pl&gt;
 * @version $Revision: 1.1 $
 * 
 */
public class Table implements Serializable {

	private static final long serialVersionUID = 5874976879105698752L;

	private Dialect dialect;

	private SessionImpl sessionImpl;

	private ClassMetadata classMetadata;

	private PersistentClass model;

	private String name;

	private Column[] columns;

	private int columnSpan;

	private String jdbcInsertSQL = "INSERT";

	private String jdbcUpdateSQL = "UPDATE";

	private String jdbcDeleteSQL = "DELETE";

	private String jdbcLockSQL = "SELECT";

	private int identifierLength;

	private int[] identifierPositions;

	public Table(ScEntityManager vendor, Class<?> persistentClass,
			ApplicationContext appCtx, Log log) throws InstantiationException,
			IllegalAccessException, ClassNotFoundException, SecurityException,
			NoSuchMethodException {

		Hashtable<String, Column> colHashTable = new Hashtable<String, Column>();
		Iterator<?> iter, subiter;
		int maxColumnNameLenght = 0;
		int maxPropertyNameLenght = 0;
		int maxTypeNameLenght = 0;
		org.hibernate.mapping.Table hibernateTable;
		org.hibernate.mapping.Column hibernateColumn;

		this.sessionImpl = (SessionImpl) vendor.getDelegate();
		SessionFactory sessionFactory = this.sessionImpl.getSessionFactory();

		this.dialect = sessionImpl.getFactory().getDialect();
		this.classMetadata = sessionFactory.getClassMetadata(persistentClass);

		this.model = vendor.getConfiguration().getClassMapping(
				classMetadata.getEntityName());
		Mapping mapping = vendor.getMapping();

		hibernateTable = this.model.getIdentityTable();
		int columnSpan = hibernateTable.getColumnSpan();
		String[] columnNames = new String[columnSpan];
		String[] aliases = new String[columnSpan];
		String[] sqlTypes = new String[columnSpan];
		String[] propertyNames = new String[columnSpan];

		Value idValue = this.model.getIdentifier();
		String propName = this.model.getIdentifierProperty().getName();
		int identifierLength = this.model.getIdentifierProperty()
				.getColumnSpan();
		iter = idValue.getColumnIterator();
		int i = 0;
		while (iter.hasNext()) {
			hibernateColumn = (org.hibernate.mapping.Column) iter.next();
			columnNames[i] = hibernateColumn.getQuotedName(dialect);
			aliases[i] = hibernateColumn.getAlias(dialect);
			sqlTypes[i] = hibernateColumn.getSqlType(dialect, mapping);
			propertyNames[i] = propName;
			maxColumnNameLenght = (maxColumnNameLenght > columnNames[i]
					.length() ? maxColumnNameLenght : columnNames[i].length());
			maxTypeNameLenght = (maxTypeNameLenght > sqlTypes[i].length() ? maxTypeNameLenght
					: sqlTypes[i].length());
			maxPropertyNameLenght = (maxPropertyNameLenght > propertyNames[i]
					.length() ? maxPropertyNameLenght : propertyNames[i]
					.length());
			i++;
		}

		iter = this.model.getPropertyIterator();
		while (iter.hasNext()) {
			Property prop = (Property) iter.next();
			subiter = prop.getColumnIterator();
			propName = prop.getName();
			while (subiter.hasNext()) {
				hibernateColumn = (org.hibernate.mapping.Column) subiter.next();
				columnNames[i] = hibernateColumn.getQuotedName(dialect);
				aliases[i] = hibernateColumn.getAlias(dialect);
				sqlTypes[i] = hibernateColumn.getSqlType(dialect, mapping);
				propertyNames[i] = propName;
				maxColumnNameLenght = (maxColumnNameLenght > columnNames[i]
						.length() ? maxColumnNameLenght : columnNames[i]
						.length());
				maxTypeNameLenght = (maxTypeNameLenght > sqlTypes[i].length() ? maxTypeNameLenght
						: sqlTypes[i].length());
				maxPropertyNameLenght = (maxPropertyNameLenght > propertyNames[i]
						.length() ? maxPropertyNameLenght : propertyNames[i]
						.length());
				i++;
			}
		}
		i = 0;

		Type itype = classMetadata.getIdentifierType();
		i = createColumnInfo(appCtx, colHashTable, columnNames, aliases,
				sqlTypes, propertyNames, i, itype, mapping, true);

		for (Type ptype : classMetadata.getPropertyTypes()) {
			i = createColumnInfo(appCtx, colHashTable, columnNames, aliases,
					sqlTypes, propertyNames, i, ptype, mapping, false);
		}

		iter = hibernateTable.getForeignKeyIterator();
		while (iter.hasNext()) {
			ForeignKey fk = (ForeignKey) iter.next();

			subiter = fk.getColumnIterator();
			while (subiter.hasNext()) {
				hibernateColumn = (org.hibernate.mapping.Column) subiter.next();
				String columnName = hibernateColumn.getQuotedName(dialect);
				Column ci = colHashTable.get(columnName);
				ci.setForeignKey(true);
			}
		}

		this.name = hibernateTable.getName();
		this.identifierLength = identifierLength;
		this.columnSpan = columnSpan;
		this.columns = new Column[columnSpan];

		Enumeration<String> keys = colHashTable.keys();
		while (keys.hasMoreElements()) {
			String key = keys.nextElement();
			Column ci = colHashTable.get(key);
			this.columns[ci.getPosition()] = ci;
		}

		int[] identifierPositions = new int[identifierLength];
		i = 0;

		Annotation[] annotations = persistentClass.getAnnotations();
		for (Annotation annotation : annotations) {
			if (annotation.annotationType().getSimpleName()
					.equals("DeleteHint")) {
				this.jdbcDeleteSQL += " /*+ "
						+ ((DeleteHint) annotation).value() + " */";
			}
			if (annotation.annotationType().getSimpleName()
					.equals("InsertHint")) {
				this.jdbcInsertSQL += " /*+ "
						+ ((InsertHint) annotation).value() + " */";
			}
			if (annotation.annotationType().getSimpleName()
					.equals("UpdateHint")) {
				this.jdbcUpdateSQL += " /*+ "
						+ ((UpdateHint) annotation).value() + " */";
			}
		}

		this.jdbcInsertSQL += " INTO " + this.name + " (";
		this.jdbcUpdateSQL += " " + this.name + " SET ";
		this.jdbcDeleteSQL += " FROM " + this.name;

		String insertValuesClause = ") values (";
		String whereClause = " WHERE ";

		for (Column ci : this.columns) {
			if (ci.isPrimaryKey()) {
				identifierPositions[i] = ci.getKeyPosition();
				i++;
				whereClause += ci.getName() + " = ? AND ";
				this.jdbcLockSQL += " " + ci.getName() + ",";
			} else {
				this.jdbcUpdateSQL += ci.getName() + "=?, ";
			}
			this.jdbcInsertSQL += ci.getName() + ", ";
			insertValuesClause += "?, ";
		}
		this.identifierPositions = identifierPositions;
		this.jdbcLockSQL = this.jdbcLockSQL.substring(0,
				this.jdbcLockSQL.length() - 1)
				+ " FROM " + this.name;
		this.jdbcInsertSQL = this.jdbcInsertSQL.substring(0,
				this.jdbcInsertSQL.length() - 2);
		this.jdbcInsertSQL += insertValuesClause.substring(0,
				insertValuesClause.length() - 2) + ")";
		whereClause = whereClause.substring(0, whereClause.length() - 5);

		this.jdbcUpdateSQL = this.jdbcUpdateSQL.substring(0,
				this.jdbcUpdateSQL.length() - 2);
		this.jdbcUpdateSQL += whereClause;
		this.jdbcDeleteSQL += whereClause;
		this.jdbcLockSQL += whereClause;

		if (log.isDebugEnabled()) {
			String tableDebugInfo = "";
			int maxLen = maxTypeNameLenght + maxColumnNameLenght
					+ maxPropertyNameLenght + 29;
			tableDebugInfo = tableDebugInfo + "\n  TABLE NAME: " + this.name;
			tableDebugInfo = tableDebugInfo + "\n  "
					+ persistentClass.getName();
			tableDebugInfo = tableDebugInfo + Strings.rpad("\n ", "-", maxLen);
			for (Column ci : this.columns) {
				tableDebugInfo = tableDebugInfo
						+ "\n | "
						+ Strings.lpad(Integer.toString(ci.getPosition()), " ",
								3)
						+ " | "
						+ Strings.lpad(Integer.toString(ci.getKeyPosition()),
								" ", 3)
						+ " | "
						+ (ci.isPrimaryKey() ? "*" : " ")
						+ (ci.isForeignKey() ? "F" : " ")
						+ " | "
						+ Strings.rpad(ci.getName(), " ", maxColumnNameLenght)
						+ " | "
						+ Strings.rpad(ci.getPropertyName(), " ",
								maxPropertyNameLenght) + " | "
						+ Strings.rpad(ci.getSqlType(), " ", maxTypeNameLenght)
						+ " | ";

			}
			tableDebugInfo = tableDebugInfo + Strings.rpad("\n ", "-", maxLen);
			tableDebugInfo = tableDebugInfo + "\n DELETE SQL: \n	"
					+ this.jdbcDeleteSQL;
			tableDebugInfo = tableDebugInfo + "\n INSERT SQL: \n	"
					+ this.jdbcInsertSQL;
			tableDebugInfo = tableDebugInfo + "\n UPDATE SQL: \n	"
					+ this.jdbcUpdateSQL;
			tableDebugInfo = tableDebugInfo + "\n LOCK SQL: \n	"
					+ this.jdbcLockSQL;

			// log.debug(tableDebugInfo);
		}

	}

	private int createColumnInfo(ApplicationContext appCtx,
			Hashtable<String, Column> colInfo, String[] columnNames,
			String[] aliases, String[] sqlTypes, String[] propertyNames, int i,
			Type itype, Mapping mapping, boolean identifier) {
		int colLength = itype.getColumnSpan(mapping);

		// TODO zrobic obsluge pozostalych typow
		// bo prawdopodonie beda potrzebne np. collection gdy bedzie
		// powiazanie z delete cascade
		if (itype.isComponentType()) {
			ComponentType com = (ComponentType) itype;
			int keyPos = 0;
			for (String prop : com.getPropertyNames()) {
				Column c = new Column();
				c.setAlias(aliases[i]);
				c.setName(columnNames[i]);
				c.setPosition(i);
				c.setSqlType(sqlTypes[i]);
				c.setPropertyName(prop);
				c.setKeyPosition(keyPos++);
				c.setType(itype);
				c.setComponentType(true);
				c.setPrimaryKey(identifier);
				c.setColumnLength(colLength);
				colInfo.put(columnNames[i], c);
				i++;
			}
		} else if (itype.isAssociationType()) {
			AssociationType com = (AssociationType) itype;
			_IJpaDAO<?> associatedDAO = null;
			boolean associated = false;
			String A = com.getReturnedClass().getName();
			for (String s : appCtx.getBeanNamesForType(_IJpaDAO.class)) {

				_IJpaDAO<?> fkDAO = (_IJpaDAO<?>) appCtx.getBean(s);
				String B = fkDAO.getPersistentClass().getName();
				if (A.equals(B)) {
					associatedDAO = fkDAO;
					associated = true;
					break;
				}
			}
			if (!associated) {
				throw new IllegalArgumentException(
						"Can't find associated bean of class '" + A
								+ "' in application context");
			}
			for (int keyPos = 0; keyPos < colLength; keyPos++) {
				Column c = new Column();
				c.setAlias(aliases[i]);
				c.setName(columnNames[i]);
				c.setPosition(i);
				c.setSqlType(sqlTypes[i]);
				c.setPropertyName(propertyNames[i]);
				c.setKeyPosition(keyPos);
				c.setType(itype);
				c.setPrimaryKey(identifier);
				c.setComponentType(false);
				c.setAnyType(itype.isAnyType());
				c.setAssociationType(itype.isAssociationType());
				c.setCollectionType(itype.isCollectionType());
				c.setEntityType(itype.isEntityType());
				c.setColumnLength(colLength);
				c.setAssociatedDAO(associatedDAO);
				colInfo.put(columnNames[i], c);
				i++;
			}
		} else {
			for (int keyPos = 0; keyPos < colLength; keyPos++) {
				Column c = new Column();
				c.setAlias(aliases[i]);
				c.setName(columnNames[i]);
				c.setPosition(i);
				c.setSqlType(sqlTypes[i]);
				c.setPropertyName(propertyNames[i]);
				c.setKeyPosition(keyPos);
				c.setType(itype);
				c.setPrimaryKey(identifier);
				c.setComponentType(false);
				c.setAnyType(itype.isAnyType());
				c.setAssociationType(itype.isAssociationType());
				c.setCollectionType(itype.isCollectionType());
				c.setEntityType(itype.isEntityType());
				c.setColumnLength(colLength);
				colInfo.put(columnNames[i], c);
				i++;
			}

		}
		return i;
	}

	public RowValues fetchValues(Object entityObject) throws SQLException {

		Object[] values = classMetadata.getPropertyValues(entityObject,
				org.hibernate.EntityMode.POJO);
		Object identifier = classMetadata.getIdentifier(entityObject,
				org.hibernate.EntityMode.POJO);

		int i = 0;
		for (Column ci : this.columns) {
			Object columnValue;
			if (ci.isPrimaryKey()) {
				columnValue = identifier;
			} else {
				columnValue = values[i];
				i = ((ci.getKeyPosition() + 1) == ci.getColumnLength() ? i + 1
						: i);
			}
			ci.fetchValue(columnValue, this.sessionImpl);
		}

		return new RowValues(this.columns, this.identifierPositions);
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the columns
	 */
	public Column[] getColumns() {
		return columns;
	}

	/**
	 * @return the columnSpan
	 */
	public int getColumnSpan() {
		return columnSpan;
	}

	/**
	 * @return the jdbcInsertSQL
	 */
	public String getJdbcInsertSQL() {
		return jdbcInsertSQL;
	}

	/**
	 * @return the jdbcUpdateSQL
	 */
	public String getJdbcUpdateSQL() {
		return jdbcUpdateSQL;
	}

	/**
	 * @return the jdbcDeleteSQL
	 */
	public String getJdbcDeleteSQL() {
		return jdbcDeleteSQL;
	}

	/**
	 * @return the identifierLength
	 */
	public int getIdentifierLength() {
		return identifierLength;
	}

	/**
	 * @return the identifierPositions
	 */
	public int[] getIdentifierPositions() {
		return identifierPositions;
	}

	/**
	 * @return the dialect
	 */
	public Dialect getDialect() {
		return dialect;
	}

	/**
	 * @return the sessionImpl
	 */
	public SessionImpl getSessionImpl() {
		return sessionImpl;
	}

	/**
	 * @return the classMetadata
	 */
	public ClassMetadata getClassMetadata() {
		return classMetadata;
	}

	/**
	 * @return the model
	 */
	public PersistentClass getModel() {
		return model;
	}

	/**
	 * @return the jdbcLockSQL
	 */
	public String getJdbcLockSQL() {
		return jdbcLockSQL;
	}

}
